#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Akini Ross <akinross@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_access_span_src_group_src_path
short_description: Manage Access SPAN source paths (span:RsSrcToPathEp)
description:
- Manage Access SPAN source paths on Cisco ACI fabrics.
options:
  source_group:
    description:
    - The name of the Access SPAN source group.
    type: str
    aliases: [ src_group ]
  source:
    description:
    - The name of the Access SPAN source.
    type: str
    aliases: [ src ]
  pod:
    description:
    - The pod id of the source access path.
    type: int
    aliases: [ pod_id, pod_number ]
  nodes:
    description:
    - The node of the source access path.
    - Provide one node to the list for port, direct_port_channel or vpc_component_pc types.
    - Provide two nodes to the list for virtual_port_channel type.
    type: list
    elements: int
    aliases: [ node_ids ]
  path_ep:
    description:
    - The path of the source access path.
    - When path is of type port a interface like C(eth1/7) must be provided.
    - When path is of type direct_port_channel the name of a pc interface policy group like C(test_PolGrp) must be provided.
    - When path is of type virtual_port_channel or vpc_component_pc the name of a vpc interface policy group like C(test_vPC_PolGrp) must be provided.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation

notes:
- The I(source_group) and I(source) must exist before using this module in your playbook.
  The M(cisco.aci.aci_access_span_src_group) and M(cisco.aci.aci_access_span_src_group_src) modules can be used for this.
seealso:
- module: cisco.aci.aci_access_span_src_group
- module: cisco.aci.aci_access_span_src_group_src
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(span:RsSrcToPathEp).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Akini Ross (@akinross)
"""

EXAMPLES = r"""
- name: Create a Access SPAN source path of type Port
  cisco.aci.aci_access_span_src_group_src_path:
    host: apic
    username: admin
    password: SomeSecretPassword
    source_group: my_span_source_group
    source: my_source
    pod: 1
    nodes:
    - 101
    path_ep: eth1/1
    state: present
  delegate_to: localhost

- name: Create a Access SPAN source path of type Direct Port Channel
  cisco.aci.aci_access_span_src_group_src_path:
    host: apic
    username: admin
    password: SomeSecretPassword
    source_group: my_span_source_group
    source: my_source
    pod: 1
    nodes:
    - 101
    path_ep: test_PolGrp
    state: present
  delegate_to: localhost

- name: Create a Access SPAN source path of type VPC component PC
  cisco.aci.aci_access_span_src_group_src_path:
    host: apic
    username: admin
    password: SomeSecretPassword
    source_group: my_span_source_group
    source: my_source
    pod: 1
    nodes:
    - 101
    path_ep: test_vPC_PolGrp
    state: present
  delegate_to: localhost

- name: Create a Access SPAN source path of type Virtual Port Channel
  cisco.aci.aci_access_span_src_group_src_path:
    host: apic
    username: admin
    password: SomeSecretPassword
    source_group: my_span_source_group
    source: my_source
    pod: 1
    nodes:
    - 101
    - 102
    path_ep: test_vPC_PolGrp
    state: present
  delegate_to: localhost

- name: Delete a Access SPAN source path
  cisco.aci.aci_access_span_src_group_src_path:
    host: apic
    username: admin
    password: SomeSecretPassword
    source_group: my_span_source_group
    source: my_source
    pod: 1
    nodes:
    - 101
    path_ep: test_vPC_PolGrp
    state: absent
  delegate_to: localhost

- name: Query all Access SPAN source paths
  cisco.aci.aci_access_span_src_group_src_path:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Query a specific Access SPAN source path
  cisco.aci.aci_access_span_src_group_src_path:
    host: apic
    username: admin
    password: SomeSecretPassword
    source_group: my_span_source_group
    source: my_source
    pod: 1
    nodes:
    - 101
    path_ep: test_vPC_PolGrp
    state: query
  delegate_to: localhost
  register: query_result
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(
        source_group=dict(type="str", aliases=["src_group"]),  # Not required for querying all objects
        source=dict(type="str", aliases=["src"]),  # Not required for querying all objects
        pod=dict(type="int", aliases=["pod_id", "pod_number"]),  # Not required for querying all objects
        nodes=dict(type="list", elements="int", aliases=["node_ids"]),  # Not required for querying all objects
        path_ep=dict(type="str"),  # Not required for querying all objects
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["source_group", "source", "pod", "nodes", "path_ep"]],
            ["state", "present", ["source_group", "source", "pod", "nodes", "path_ep"]],
        ],
    )

    aci = ACIModule(module)

    source_group = module.params.get("source_group")
    source = module.params.get("source")
    pod = module.params.get("pod")
    nodes = module.params.get("nodes")
    path_ep = module.params.get("path_ep")
    state = module.params.get("state")

    tdn = None
    if nodes:
        if len(nodes) == 1:
            tdn = "topology/pod-{0}/paths-{1}/pathep-[{2}]".format(pod, nodes[0], path_ep)
        elif len(nodes) == 2:
            tdn = "topology/pod-{0}/protpaths-{1}/pathep-[{2}]".format(pod, "-".join([str(node) for node in nodes]), path_ep)
        else:
            module.fail_json(msg="{0} nodes have been provided, where a maximum of 2 nodes is allowed.".format(len(nodes)))

    aci.construct_url(
        root_class=dict(
            aci_class="infra",
            aci_rn="infra",
        ),
        subclass_1=dict(
            aci_class="spanSrcGrp",
            aci_rn="srcgrp-{0}".format(source_group),
            module_object=source_group,
            target_filter={"name": source_group},
        ),
        subclass_2=dict(
            aci_class="spanSrc",
            aci_rn="src-{0}".format(source),
            module_object=source,
            target_filter={"name": source},
        ),
        subclass_3=dict(
            aci_class="spanRsSrcToPathEp",
            aci_rn="rssrcToPathEp-[{0}]".format(tdn),
            module_object=tdn,
            target_filter={"tDn": tdn},
        ),
    )

    aci.get_existing()

    if state == "present":
        aci.payload(aci_class="spanRsSrcToPathEp", class_config=dict(tDn=tdn))

        aci.get_diff(aci_class="spanRsSrcToPathEp")

        aci.post_config()

    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2022 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class TemplatedOptionProvider(object):
    def __init__(self, plugin, templar):
        self.plugin = plugin
        self.templar = templar

    def get_option(self, option_name):
        value = self.plugin.get_option(option_name)
        if self.templar.is_template(value):
            value = self.templar.template(variable=value, disable_lookups=False)
        return value



#!/usr/bin/python
#
# Copyright (c) 2022 Ross Bender (@l3ender)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_containerregistrytag
version_added: "1.12.0"
short_description: Import or delete tags in Azure Container Registry
description:
    - Import or delete tags in Azure Container Registry.

options:
    resource_group:
        description:
            - The resource group of the registry.
        type: str
    registry:
        description:
            - The name of the container registry.
        type: str
        required: true
    repository_name:
        description:
            - The name of the repository within the registry.
            - Required when state = C(absent).
            - If omitted when I(state=present), the name of the source repository will be used.
        type: str
    name:
        description:
            - The name of the tag.
            - If omitted when I(state=present), the name of the source tag will be used.
            - If omitted when I(state=absent), the whole repository will be deleted.
        type: str
    source_image:
        description:
            - The source image detail. Required when I(state=present).
        type: dict
        suboptions:
            registry_uri:
                description:
                    - The address of the source registry.
                type: str
            repository:
                description:
                    - Repository name of the source image.
                type: str
                required: true
            name:
                description:
                    - Name of the tag.
                type: str
                default: latest
            credentials:
                description:
                    - Credentials for the source registry.
                type: dict
                suboptions:
                    username:
                        description:
                            - Username for the source registry.
                        type: str
                    password:
                        description:
                            - Password for the source registry.
                        type: str
    state:
        description:
            - State of the container registry tag.
            - Use C(present) to create or update a  container registry tag and use C(absent) to delete an  container registry tag.
        type: str
        default: present
        choices:
            - present
            - absent

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Ross Bender (@l3ender)
'''

EXAMPLES = '''
- name: Import a tag
  azure_rm_containerregistrytag:
    registry: myRegistry
    source_image:
      registry_uri: docker.io
      repository: library/hello-world
      name: latest

- name: Import a tag to a different name
  azure_rm_containerregistrytag:
    registry: myRegistry
    repository_name: app1
    name: v1
    source_image:
      registry_uri: docker.io
      repository: library/hello-world
      name: latest

- name: Delete all tags in repository
  azure_rm_containerregistrytag:
    registry: myRegistry
    repository_name: myRepository
    state: absent

- name: Delete specific tag in repository
  azure_rm_containerregistrytag:
    registry: myRegistry
    repository_name: myRepository
    name: myTag
    state: absent
'''

RETURN = '''
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase, azure_id_to_dict

try:
    from azure.mgmt.containerregistry.models import ImportImageParameters, ImportSource, ImportSourceCredentials
    from azure.containerregistry import ContainerRegistryClient
except ImportError as exc:
    # This is handled in azure_rm_common
    pass


class Actions:
    NoAction, Import, DeleteRepo, DeleteTag = range(4)


class AzureRMContainerRegistryTag(AzureRMModuleBase):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type="str",
            ),
            registry=dict(
                type="str",
                required=True,
            ),
            repository_name=dict(
                type="str",
            ),
            name=dict(
                type="str",
            ),
            source_image=dict(
                type="dict",
                options=dict(
                    registry_uri=dict(
                        type="str",
                    ),
                    repository=dict(
                        type="str",
                        required=True,
                    ),
                    name=dict(
                        type="str",
                        default="latest",
                    ),
                    credentials=dict(
                        type="dict",
                        options=dict(
                            username=dict(type="str"),
                            password=dict(type="str", no_log=True),
                        )
                    ),
                ),
            ),
            state=dict(
                type="str",
                default="present",
                choices=["present", "absent"],
            )
        )

        required_if = [
            ("state", "present", ["source_image"]),
            ("state", "absent", ["repository_name"]),
        ]

        self.results = dict(
            changed=True
        )

        self.resource_group = None
        self.registry = None
        self.repository_name = None
        self.name = None
        self.source_image = None
        self.state = None

        self._client = None
        self._todo = Actions.NoAction

        super(AzureRMContainerRegistryTag, self).__init__(self.module_arg_spec,
                                                          supports_check_mode=True,
                                                          supports_tags=False,
                                                          facts_module=False,
                                                          required_if=required_if)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        self._client = self.get_client()

        if self.state == "present":
            repo_name = self.repository_name if self.repository_name else self.source_image["repository"]
            tag_name = self.name if self.name else self.source_image["name"]
            tag = self.get_tag(repo_name, tag_name)
            if not tag:
                self._todo = Actions.Import
        elif self.state == "absent":
            if self.repository_name and self.name:
                tag = self.get_tag(self.repository_name, self.name)
                if tag:
                    self._todo = Actions.DeleteTag
            else:
                repository = self.get_repository(self.repository_name)
                if repository:
                    self._todo = Actions.DeleteRepo

        if self._todo == Actions.Import:
            self.log("importing image into registry")
            if not self.check_mode:
                self.import_tag(self.repository_name, self.name, self.resource_group, self.registry, self.source_image)
        elif self._todo == Actions.DeleteTag:
            self.log(f"deleting tag {self.repository_name}:{self.name}")
            if not self.check_mode:
                self.delete_tag(self.repository_name, self.name)
        elif self._todo == Actions.DeleteRepo:
            self.log(f"deleting repository {self.repository_name}")
            if not self.check_mode:
                self.delete_repository(self.repository_name)
        else:
            self.log("no action")
            self.results["changed"] = False

        return self.results

    def get_client(self):
        registry_endpoint = self.registry if self.registry.endswith(".azurecr.io") else self.registry + ".azurecr.io"
        return ContainerRegistryClient(
            endpoint=registry_endpoint,
            credential=self.azure_auth.azure_credential_track2,
            audience="https://management.azure.com",
        )

    def get_repository(self, repository_name):
        response = None
        try:
            response = self._client.get_repository_properties(repository=repository_name)
            self.log(f"Response : {response}")
        except Exception as e:
            self.log(f"Could not get ACR repository for {repository_name} - {str(e)}")

        if response is not None:
            return response.name

        return None

    def get_tag(self, repository_name, tag_name):
        response = None
        try:
            self.log(f"Getting tag for {repository_name}:{tag_name}")
            response = self._client.get_tag_properties(repository=repository_name, tag=tag_name)
            self.log(f"Response : {response}")
        except Exception as e:
            self.log(f"Could not get ACR tag for {repository_name}:{tag_name} - {str(e)}")

        return response

    def import_tag(self, repository, tag, resource_group, registry, source_image):
        source_tag = get_tag(source_image["repository"], source_image["name"])
        dest_repo_name = repository if repository else source_image["repository"]
        dest_tag_name = tag if tag else source_image["name"]
        dest_tag = get_tag(dest_repo_name, dest_tag_name)
        creds = None if not source_image["credentials"] else ImportSourceCredentials(
            username=source_image["credentials"]["username"],
            password=source_image["credentials"]["password"],
        )
        params = ImportImageParameters(
            target_tags=[dest_tag],
            source=ImportSource(
                registry_uri=source_image["registry_uri"],
                source_image=source_tag,
                credentials=creds,
            )
        )
        try:
            if not resource_group:
                resource_group = self.get_registry_resource_group(registry)

            self.log(f"Importing {source_tag} as {dest_tag} to {registry} in {resource_group}")
            poller = self.containerregistry_client.registries.begin_import_image(resource_group_name=resource_group,
                                                                                 registry_name=registry,
                                                                                 parameters=params)
            self.get_poller_result(poller)
        except Exception as e:
            self.fail(f"Could not import {source_tag} as {dest_tag} to {registry} in {resource_group} - {str(e)}")

    def get_registry_resource_group(self, registry_name):
        response = None
        try:
            response = self.containerregistry_client.registries.list()
        except Exception as e:
            self.fail(f"Could not load resource group for registry {registry_name} - {str(e)}")

        if response is not None:
            for item in response:
                item_dict = item.as_dict()
                if item_dict["name"] == registry_name:
                    return azure_id_to_dict(item_dict["id"]).get("resourceGroups")

        return None

    def delete_repository(self, repository_name):
        try:
            self._client.delete_repository(repository=repository_name)
        except Exception as e:
            self.fail(f"Could not delete repository {repository_name} - {str(e)}")

    def delete_tag(self, repository_name, tag_name):
        try:
            self._client.delete_tag(repository=repository_name, tag=tag_name)
        except Exception as e:
            self.fail(f"Could not delete tag {repository_name}:{tag_name} - {str(e)}")


def get_tag(repository, tag):
    return repository if not tag else repository + ":" + tag


def main():
    AzureRMContainerRegistryTag()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_fabric_authentication_profile
short_description: Resource module for Sda Fabric Authentication Profile
description:
- Manage operations create, update and delete of the resource Sda Fabric Authentication Profile.
- Add default authentication template in SDA Fabric.
- Delete default authentication profile in SDA Fabric.
- Update default authentication profile in SDA Fabric.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  payload:
    description: Sda Fabric Authentication Profile's payload.
    elements: dict
    suboptions:
      authenticateTemplateName:
        description: Authenticate Template Name.
        type: str
        version_added: 4.0.0
      siteNameHierarchy:
        description: Path of sda Fabric Site.
        type: str
    type: list
  siteNameHierarchy:
    description: SiteNameHierarchy query parameter.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA AddDefaultAuthenticationTemplateInSDAFabric
  description: Complete reference of the AddDefaultAuthenticationTemplateInSDAFabric API.
  link: https://developer.cisco.com/docs/dna-center/#!add-default-authentication-template-in-sda-fabric
- name: Cisco DNA Center documentation for SDA DeleteDefaultAuthenticationProfileFromSDAFabric
  description: Complete reference of the DeleteDefaultAuthenticationProfileFromSDAFabric API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-default-authentication-profile-from-sda-fabric
- name: Cisco DNA Center documentation for SDA UpdateDefaultAuthenticationProfileInSDAFabric
  description: Complete reference of the UpdateDefaultAuthenticationProfileInSDAFabric API.
  link: https://developer.cisco.com/docs/dna-center/#!update-default-authentication-profile-in-sda-fabric
notes:
  - SDK Method used are
    sda.Sda.add_default_authentication_profile,
    sda.Sda.delete_default_authentication_profile,
    sda.Sda.update_default_authentication_profile,

  - Paths used are
    post /dna/intent/api/v1/business/sda/authentication-profile,
    delete /dna/intent/api/v1/business/sda/authentication-profile,
    put /dna/intent/api/v1/business/sda/authentication-profile,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.sda_fabric_authentication_profile:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - authenticateTemplateName: string
      siteNameHierarchy: string

- name: Update all
  cisco.dnac.sda_fabric_authentication_profile:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - authenticateTemplateName: string
      authenticationOrder: string
      dot1xToMabFallbackTimeout: string
      numberOfHosts: string
      siteNameHierarchy: string
      wakeOnLan: true

- name: Delete all
  cisco.dnac.sda_fabric_authentication_profile:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    siteNameHierarchy: string

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "status": "string",
      "description": "string",
      "taskId": "string",
      "taskStatusUrl": "string",
      "executionStatusUrl": "string",
      "executionId": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_ntp_policy
short_description: NTP policy configuration for Cisco Intersight
description:
  - NTP policy configuration for Cisco Intersight.
  - Used to configure NTP servers and timezone settings on Cisco Intersight managed devices.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles and Policies that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the NTP policy.
      - The name must be between 1 and 62 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  description:
    description:
      - The user-defined description of the NTP policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  enable:
    description:
      - Enable or disable NTP.
    type: bool
    default: true
  ntp_servers:
    description:
      - List of NTP servers configured on the endpoint.
    type: list
    elements: str
  timezone:
    description:
      - Timezone of services on the endpoint.
    type: str
author:
  - David Soper (@dsoper2)
'''

EXAMPLES = r'''
- name: Configure NTP Policy
  cisco.intersight.intersight_ntp_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: DevNet
    name: lab-ntp
    description: NTP policy for lab use
    tags:
      - Key: Site
        Value: RCDN
    ntp_servers:
      - ntp.esl.cisco.com
    timezone: America/Chicago

- name: Delete NTP Policy
  cisco.intersight.intersight_ntp_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: DevNet
    name: lab-ntp
    state: absent
'''

RETURN = r'''
api_repsonse:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "lab-ntp",
        "ObjectType": "ntp.Policy",
        "Tags": [
            {
                "Key": "Site",
                "Value": "RCDN"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def main():
    argument_spec = intersight_argument_spec
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        enable=dict(type='bool', default=True),
        ntp_servers=dict(type='list', elements='str'),
        timezone=dict(type='str'),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''
    #
    # Argument spec above, resource path, and API body should be the only code changed in each policy module
    #
    # Resource path used to configure policy
    resource_path = '/ntp/Policies'
    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name'],
        'Tags': intersight.module.params['tags'],
        'Description': intersight.module.params['description'],
        'Enabled': intersight.module.params['enable'],
        'NtpServers': intersight.module.params['ntp_servers'],
        'Timezone': intersight.module.params['timezone'],
    }

    #
    # Code below should be common across all policy modules
    #
    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sensor_test_run
short_description: Resource module for Sensor Test Run
description:
- Manage operation update of the resource Sensor Test Run.
- Intent API to run a deployed SENSOR test.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  templateName:
    description: Template Name.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Sensors RunNowSensorTest
  description: Complete reference of the RunNowSensorTest API.
  link: https://developer.cisco.com/docs/dna-center/#!run-now-sensor-test
notes:
  - SDK Method used are
    sensors.Sensors.run_now_sensor_test,

  - Paths used are
    put /dna/intent/api/v1/sensor-run-now,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.sensor_test_run:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    templateName: string

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: certificate_profile_info
short_description: Information module for Certificate Profile
description:
- Get all Certificate Profile.
- Get Certificate Profile by id.
- Get Certificate Profile by name.
- This API allows the client to get a certificate profile by ID.
- This API allows the client to get a certificate profile by name.
- This API allows the client to get all the certificate profiles.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  name:
    description:
    - Name path parameter.
    type: str
  id:
    description:
    - Id path parameter.
    type: str
  page:
    description:
    - Page query parameter. Page number.
    type: int
  size:
    description:
    - Size query parameter. Number of objects returned per page.
    type: int
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
notes:
  - SDK Method used are
    certificate_profile.CertificateProfile.get_certificate_profile_by_id,
    certificate_profile.CertificateProfile.get_certificate_profile_by_name,
    certificate_profile.CertificateProfile.get_certificate_profile_generator,

  - Paths used are
    get /ers/config/certificateprofile,
    get /ers/config/certificateprofile/name/{name},
    get /ers/config/certificateprofile/{id},

"""

EXAMPLES = r"""
- name: Get all Certificate Profile
  cisco.ise.certificate_profile_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    page: 1
    size: 20
  register: result

- name: Get Certificate Profile by id
  cisco.ise.certificate_profile_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    id: string
  register: result

- name: Get Certificate Profile by name
  cisco.ise.certificate_profile_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    name: string
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "id": "string",
      "name": "string",
      "description": "string",
      "externalIdentityStoreName": "string",
      "certificateAttributeName": "string",
      "allowedAsUserName": true,
      "matchMode": "string",
      "usernameFrom": "string",
      "link": {
        "rel": "string",
        "href": "string",
        "type": "string"
      }
    }

ise_responses:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  version_added: '1.1.0'
  type: list
  elements: dict
  sample: >
    [
      {
        "id": "string",
        "name": "string",
        "description": "string",
        "externalIdentityStoreName": "string",
        "certificateAttributeName": "string",
        "allowedAsUserName": true,
        "matchMode": "string",
        "usernameFrom": "string",
        "link": {
          "rel": "string",
          "href": "string",
          "type": "string"
        }
      }
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: lightsail
version_added: 1.0.0
short_description: Manage instances in AWS Lightsail
description:
  - Manage instances in AWS Lightsail.
  - Instance tagging is not yet supported in this module.
author:
  - "Nick Ball (@nickball)"
  - "Prasad Katti (@prasadkatti)"
options:
  state:
    description:
      - Indicate desired state of the target.
      - I(rebooted) and I(restarted) are aliases.
    default: present
    choices: ['present', 'absent', 'running', 'restarted', 'rebooted', 'stopped']
    type: str
  name:
    description: Name of the instance.
    required: true
    type: str
  zone:
    description:
      - AWS availability zone in which to launch the instance.
      - Required when I(state=present)
    type: str
  blueprint_id:
    description:
      - ID of the instance blueprint image.
      - Required when I(state=present)
    type: str
  bundle_id:
    description:
      - Bundle of specification info for the instance.
      - Required when I(state=present).
    type: str
  user_data:
    description:
      - Launch script that can configure the instance with additional data.
    type: str
    default: ''
  public_ports:
    description:
      - A list of dictionaries to describe the ports to open for the specified instance.
    type: list
    elements: dict
    suboptions:
      from_port:
        description: The first port in a range of open ports on the instance.
        type: int
        required: true
      to_port:
        description: The last port in a range of open ports on the instance.
        type: int
        required: true
      protocol:
        description: The IP protocol name accepted for the defined range of open ports.
        type: str
        choices: ['tcp', 'all', 'udp', 'icmp']
        required: true
      cidrs:
        description:
          - The IPv4 address, or range of IPv4 addresses (in CIDR notation) that are allowed to connect to the instance through the ports, and the protocol.
          - One of I(cidrs) or I(ipv6_cidrs) must be specified.
        type: list
        elements: str
      ipv6_cidrs:
        description:
          - The IPv6 address, or range of IPv6 addresses (in CIDR notation) that are allowed to connect to the instance through the ports, and the protocol.
          - One of I(cidrs) or I(ipv6_cidrs) must be specified.
        type: list
        elements: str
    version_added: 6.0.0
  key_pair_name:
    description:
      - Name of the key pair to use with the instance.
      - If I(state=present) and a key_pair_name is not provided, the default keypair from the region will be used.
    type: str
  wait:
    description:
      - Wait for the instance to be in state 'running' before returning.
      - If I(wait=false) an ip_address may not be returned.
      - Has no effect when I(state=rebooted) or I(state=absent).
    type: bool
    default: true
  wait_timeout:
    description:
      - How long before I(wait) gives up, in seconds.
    default: 300
    type: int

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""


EXAMPLES = r"""
- name: Create a new Lightsail instance
  community.aws.lightsail:
    state: present
    name: my_instance
    region: us-east-1
    zone: us-east-1a
    blueprint_id: ubuntu_16_04
    bundle_id: nano_1_0
    key_pair_name: id_rsa
    user_data: " echo 'hello world' > /home/ubuntu/test.txt"
    public_ports:
      - from_port: 22
        to_port: 22
        protocol: "tcp"
        cidrs: ["0.0.0.0/0"]
        ipv6_cidrs: ["::/0"]
  register: my_instance

- name: Delete an instance
  community.aws.lightsail:
    state: absent
    region: us-east-1
    name: my_instance
"""

RETURN = r"""
changed:
  description: if a snapshot has been modified/created
  returned: always
  type: bool
  sample:
    changed: true
instance:
  description: instance data
  returned: always
  type: dict
  sample:
    arn: "arn:aws:lightsail:us-east-1:123456789012:Instance/1fef0175-d6c8-480e-84fa-214f969cda87"
    blueprint_id: "ubuntu_16_04"
    blueprint_name: "Ubuntu"
    bundle_id: "nano_1_0"
    created_at: "2017-03-27T08:38:59.714000-04:00"
    hardware:
      cpu_count: 1
      ram_size_in_gb: 0.5
    is_static_ip: false
    location:
      availability_zone: "us-east-1a"
      region_name: "us-east-1"
    name: "my_instance"
    networking:
      monthly_transfer:
        gb_per_month_allocated: 1024
      ports:
        - access_direction: "inbound"
          access_from: "Anywhere (0.0.0.0/0)"
          access_type: "public"
          common_name: ""
          from_port: 80
          protocol: tcp
          to_port: 80
        - access_direction: "inbound"
          access_from: "Anywhere (0.0.0.0/0)"
          access_type: "public"
          common_name: ""
          from_port: 22
          protocol: tcp
          to_port: 22
    private_ip_address: "172.26.8.14"
    public_ip_address: "34.207.152.202"
    resource_type: "Instance"
    ssh_key_name: "keypair"
    state:
      code: 16
      name: running
    support_code: "123456789012/i-0997c97831ee21e33"
    username: "ubuntu"
"""

import time

try:
    import botocore
except ImportError:
    # will be caught by AnsibleAWSModule
    pass

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.botocore import is_boto3_error_code

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def find_instance_info(module, client, instance_name, fail_if_not_found=False):
    try:
        res = client.get_instance(instanceName=instance_name)
    except is_boto3_error_code("NotFoundException") as e:
        if fail_if_not_found:
            module.fail_json_aws(e)
        return None
    except botocore.exceptions.ClientError as e:  # pylint: disable=duplicate-except
        module.fail_json_aws(e)
    return res["instance"]


def wait_for_instance_state(module, client, instance_name, states):
    """
    `states` is a list of instance states that we are waiting for.
    """

    wait_timeout = module.params.get("wait_timeout")
    wait_max = time.time() + wait_timeout
    while wait_max > time.time():
        try:
            instance = find_instance_info(module, client, instance_name)
            if instance["state"]["name"] in states:
                break
            time.sleep(5)
        except botocore.exceptions.ClientError as e:
            module.fail_json_aws(e)
    else:
        module.fail_json(
            msg=f'Timed out waiting for instance "{instance_name}" to get to one of the following states - {states}'
        )


def update_public_ports(module, client, instance_name):
    try:
        client.put_instance_public_ports(
            portInfos=snake_dict_to_camel_dict(module.params.get("public_ports")),
            instanceName=instance_name,
        )
    except botocore.exceptions.ClientError as e:
        module.fail_json_aws(e)


def create_or_update_instance(module, client, instance_name):
    inst = find_instance_info(module, client, instance_name)

    if not inst:
        create_params = {
            "instanceNames": [instance_name],
            "availabilityZone": module.params.get("zone"),
            "blueprintId": module.params.get("blueprint_id"),
            "bundleId": module.params.get("bundle_id"),
            "userData": module.params.get("user_data"),
        }

        key_pair_name = module.params.get("key_pair_name")
        if key_pair_name:
            create_params["keyPairName"] = key_pair_name

        try:
            client.create_instances(**create_params)
        except botocore.exceptions.ClientError as e:
            module.fail_json_aws(e)

        wait = module.params.get("wait")
        if wait:
            desired_states = ["running"]
            wait_for_instance_state(module, client, instance_name, desired_states)

    if module.params.get("public_ports") is not None:
        update_public_ports(module, client, instance_name)
    after_update_inst = find_instance_info(module, client, instance_name, fail_if_not_found=True)

    module.exit_json(
        changed=after_update_inst != inst,
        instance=camel_dict_to_snake_dict(after_update_inst),
    )


def delete_instance(module, client, instance_name):
    changed = False

    inst = find_instance_info(module, client, instance_name)
    if inst is None:
        module.exit_json(changed=changed, instance={})

    # Wait for instance to exit transition state before deleting
    desired_states = ["running", "stopped"]
    wait_for_instance_state(module, client, instance_name, desired_states)

    try:
        client.delete_instance(instanceName=instance_name)
        changed = True
    except botocore.exceptions.ClientError as e:
        module.fail_json_aws(e)

    module.exit_json(changed=changed, instance=camel_dict_to_snake_dict(inst))


def restart_instance(module, client, instance_name):
    """
    Reboot an existing instance
    Wait will not apply here as this is an OS-level operation
    """

    changed = False

    inst = find_instance_info(module, client, instance_name, fail_if_not_found=True)

    try:
        client.reboot_instance(instanceName=instance_name)
        changed = True
    except botocore.exceptions.ClientError as e:
        module.fail_json_aws(e)

    module.exit_json(changed=changed, instance=camel_dict_to_snake_dict(inst))


def start_or_stop_instance(module, client, instance_name, state):
    """
    Start or stop an existing instance
    """

    changed = False

    inst = find_instance_info(module, client, instance_name, fail_if_not_found=True)

    # Wait for instance to exit transition state before state change
    desired_states = ["running", "stopped"]
    wait_for_instance_state(module, client, instance_name, desired_states)

    # Try state change
    if inst and inst["state"]["name"] != state:
        try:
            if state == "running":
                client.start_instance(instanceName=instance_name)
            else:
                client.stop_instance(instanceName=instance_name)
        except botocore.exceptions.ClientError as e:
            module.fail_json_aws(e)
        changed = True
        # Grab current instance info
        inst = find_instance_info(module, client, instance_name)

    wait = module.params.get("wait")
    if wait:
        desired_states = [state]
        wait_for_instance_state(module, client, instance_name, desired_states)
        inst = find_instance_info(module, client, instance_name, fail_if_not_found=True)

    module.exit_json(changed=changed, instance=camel_dict_to_snake_dict(inst))


def main():
    argument_spec = dict(
        name=dict(type="str", required=True),
        state=dict(
            type="str", default="present", choices=["present", "absent", "stopped", "running", "restarted", "rebooted"]
        ),
        zone=dict(type="str"),
        blueprint_id=dict(type="str"),
        bundle_id=dict(type="str"),
        key_pair_name=dict(type="str"),
        user_data=dict(type="str", default=""),
        wait=dict(type="bool", default=True),
        wait_timeout=dict(default=300, type="int"),
        public_ports=dict(
            type="list",
            elements="dict",
            options=dict(
                from_port=dict(type="int", required=True),
                to_port=dict(type="int", required=True),
                protocol=dict(type="str", choices=["tcp", "all", "udp", "icmp"], required=True),
                cidrs=dict(type="list", elements="str"),
                ipv6_cidrs=dict(type="list", elements="str"),
            ),
            required_one_of=[("cidrs", "ipv6_cidrs")],
        ),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec, required_if=[["state", "present", ("zone", "blueprint_id", "bundle_id")]]
    )

    client = module.client("lightsail")

    name = module.params.get("name")
    state = module.params.get("state")

    if state == "present":
        create_or_update_instance(module, client, name)
    elif state == "absent":
        delete_instance(module, client, name)
    elif state in ("running", "stopped"):
        start_or_stop_instance(module, client, name, state)
    elif state in ("restarted", "rebooted"):
        restart_instance(module, client, name)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: route53_zone
short_description: add or delete Route53 zones
version_added: 5.0.0
description:
    - Creates and deletes Route53 private and public zones.
    - This module was originally added to C(community.aws) in release 1.0.0.
options:
    zone:
        description:
            - "The DNS zone record (eg: foo.com.)"
        required: true
        type: str
    state:
        description:
            - Whether or not the zone should exist or not.
        default: present
        choices: [ "present", "absent" ]
        type: str
    vpc_id:
        description:
            - The VPC ID the zone should be a part of (if this is going to be a private zone).
        type: str
    vpc_region:
        description:
            - The VPC Region the zone should be a part of (if this is going to be a private zone).
        type: str
    vpcs:
        version_added: 5.3.0
        description:
            - The VPCs the zone should be a part of (if this is going to be a private zone).
        type: list
        elements: dict
        suboptions:
            id:
                description:
                    - The ID of the VPC.
                type: str
                required: true
            region:
                description:
                    - The region of the VPC.
                type: str
                required: true
    comment:
        description:
            - Comment associated with the zone.
        default: ''
        type: str
    hosted_zone_id:
        description:
            - The unique zone identifier you want to delete or "all" if there are many zones with the same domain name.
            - Required if there are multiple zones identified with the above options.
        type: str
    delegation_set_id:
        description:
            - The reusable delegation set ID to be associated with the zone.
            - Note that you can't associate a reusable delegation set with a private hosted zone.
        type: str
extends_documentation_fragment:
    - amazon.aws.common.modules
    - amazon.aws.region.modules
    - amazon.aws.tags
    - amazon.aws.boto3
notes:
    - Support for I(tags) and I(purge_tags) was added in release 2.1.0.
author:
    - "Christopher Troup (@minichate)"
"""

EXAMPLES = r"""
- name: create a public zone
  amazon.aws.route53_zone:
    zone: example.com
    comment: this is an example

- name: delete a public zone
  amazon.aws.route53_zone:
    zone: example.com
    state: absent

- name: create a private zone
  amazon.aws.route53_zone:
    zone: devel.example.com
    vpc_id: '{{ myvpc_id }}'
    vpc_region: us-west-2
    comment: developer domain

- name: create a private zone with multiple associated VPCs
  amazon.aws.route53_zone:
    zone: crossdevel.example.com
    vpcs:
      - id: vpc-123456
        region: us-west-2
      - id: vpc-000001
        region: us-west-2
    comment: developer cross-vpc domain

- name: create a public zone associated with a specific reusable delegation set
  amazon.aws.route53_zone:
    zone: example.com
    comment: reusable delegation set example
    delegation_set_id: A1BCDEF2GHIJKL

- name: create a public zone with tags
  amazon.aws.route53_zone:
    zone: example.com
    comment: this is an example
    tags:
      Owner: Ansible Team

- name: modify a public zone, removing all previous tags and adding a new one
  amazon.aws.route53_zone:
    zone: example.com
    comment: this is an example
    tags:
      Support: Ansible Community
    purge_tags: true
"""

RETURN = r"""
comment:
    description: optional hosted zone comment
    returned: when hosted zone exists
    type: str
    sample: "Private zone"
name:
    description: hosted zone name
    returned: when hosted zone exists
    type: str
    sample: "private.local."
private_zone:
    description: whether hosted zone is private or public
    returned: when hosted zone exists
    type: bool
    sample: true
vpc_id:
    description: id of the first vpc attached to private hosted zone (use vpcs for associating multiple).
    returned: for private hosted zone
    type: str
    sample: "vpc-1d36c84f"
vpc_region:
    description: region of the first vpc attached to private hosted zone (use vpcs for assocaiting multiple).
    returned: for private hosted zone
    type: str
    sample: "eu-west-1"
vpcs:
    version_added: 5.3.0
    description: The list of VPCs attached to the private hosted zone
    returned: for private hosted zone
    type: list
    elements: dict
    sample: "[{'id': 'vpc-123456', 'region': 'us-west-2'}]"
    contains:
        id:
            description: ID of the VPC
            returned: for private hosted zone
            type: str
            sample: "vpc-123456"
        region:
            description: Region of the VPC
            returned: for private hosted zone
            type: str
            sample: "eu-west-2"
zone_id:
    description: hosted zone id
    returned: when hosted zone exists
    type: str
    sample: "Z6JQG9820BEFMW"
delegation_set_id:
    description: id of the associated reusable delegation set
    returned: for public hosted zones, if they have been associated with a reusable delegation set
    type: str
    sample: "A1BCDEF2GHIJKL"
tags:
    description: tags associated with the zone
    returned: when tags are defined
    type: dict
"""

import time

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry
from ansible_collections.amazon.aws.plugins.module_utils.route53 import get_tags
from ansible_collections.amazon.aws.plugins.module_utils.route53 import manage_tags

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # caught by AnsibleAWSModule


@AWSRetry.jittered_backoff()
def _list_zones():
    paginator = client.get_paginator("list_hosted_zones")
    return paginator.paginate().build_full_result()


def find_zones(zone_in, private_zone):
    try:
        results = _list_zones()
    except (BotoCoreError, ClientError) as e:
        module.fail_json_aws(e, msg="Could not list current hosted zones")
    zones = []
    for r53zone in results["HostedZones"]:
        if r53zone["Name"] != zone_in:
            continue
        # only save zone names that match the public/private setting
        if (r53zone["Config"]["PrivateZone"] and private_zone) or (
            not r53zone["Config"]["PrivateZone"] and not private_zone
        ):
            zones.append(r53zone)

    return zones


def create(matching_zones):
    zone_in = module.params.get("zone").lower()
    vpc_id = module.params.get("vpc_id")
    vpc_region = module.params.get("vpc_region")
    vpcs = module.params.get("vpcs") or ([{"id": vpc_id, "region": vpc_region}] if vpc_id and vpc_region else None)
    comment = module.params.get("comment")
    delegation_set_id = module.params.get("delegation_set_id")
    tags = module.params.get("tags")
    purge_tags = module.params.get("purge_tags")

    if not zone_in.endswith("."):
        zone_in += "."

    private_zone = bool(vpcs)

    record = {
        "private_zone": private_zone,
        "vpc_id": vpcs and vpcs[0]["id"],  # The first one for backwards compatibility
        "vpc_region": vpcs and vpcs[0]["region"],  # The first one for backwards compatibility
        "vpcs": vpcs,
        "comment": comment,
        "name": zone_in,
        "delegation_set_id": delegation_set_id,
        "zone_id": None,
    }

    if private_zone:
        changed, result = create_or_update_private(matching_zones, record)
    else:
        changed, result = create_or_update_public(matching_zones, record)

    zone_id = result.get("zone_id")
    if zone_id:
        if tags is not None:
            changed |= manage_tags(module, client, "hostedzone", zone_id, tags, purge_tags)
        result["tags"] = get_tags(module, client, "hostedzone", zone_id)
    else:
        result["tags"] = tags

    return changed, result


def create_or_update_private(matching_zones, record):
    for z in matching_zones:
        try:
            result = client.get_hosted_zone(Id=z["Id"])  # could be in different regions or have different VPCids
        except (BotoCoreError, ClientError) as e:
            module.fail_json_aws(e, msg=f"Could not get details about hosted zone {z['Id']}")
        zone_details = result["HostedZone"]
        vpc_details = result["VPCs"]
        current_vpc_ids = None
        current_vpc_regions = None
        matching = False
        if isinstance(vpc_details, dict) and len(record["vpcs"]) == 1:
            if vpc_details["VPC"]["VPCId"] == record["vpcs"][0]["id"]:
                current_vpc_ids = [vpc_details["VPC"]["VPCId"]]
                current_vpc_regions = [vpc_details["VPC"]["VPCRegion"]]
                matching = True
        else:
            # Sort the lists and compare them to make sure they contain the same items
            if sorted([vpc["id"] for vpc in record["vpcs"]]) == sorted([v["VPCId"] for v in vpc_details]) and sorted(
                [vpc["region"] for vpc in record["vpcs"]]
            ) == sorted([v["VPCRegion"] for v in vpc_details]):
                current_vpc_ids = [vpc["id"] for vpc in record["vpcs"]]
                current_vpc_regions = [vpc["region"] for vpc in record["vpcs"]]
                matching = True

        if matching:
            record["zone_id"] = zone_details["Id"].replace("/hostedzone/", "")
            if "Comment" in zone_details["Config"] and zone_details["Config"]["Comment"] != record["comment"]:
                if not module.check_mode:
                    try:
                        client.update_hosted_zone_comment(Id=zone_details["Id"], Comment=record["comment"])
                    except (BotoCoreError, ClientError) as e:
                        module.fail_json_aws(e, msg=f"Could not update comment for hosted zone {zone_details['Id']}")
                return True, record
            else:
                record["msg"] = (
                    "There is already a private hosted zone in the same region with the same VPC(s)"
                    " you chose. Unable to create a new private hosted zone in the same name space."
                )
                return False, record

    if not module.check_mode:
        try:
            result = client.create_hosted_zone(
                Name=record["name"],
                HostedZoneConfig={
                    "Comment": record["comment"] if record["comment"] is not None else "",
                    "PrivateZone": True,
                },
                VPC={
                    "VPCRegion": record["vpcs"][0]["region"],
                    "VPCId": record["vpcs"][0]["id"],
                },
                CallerReference=f"{record['name']}-{time.time()}",
            )
        except (BotoCoreError, ClientError) as e:
            module.fail_json_aws(e, msg="Could not create hosted zone")

        hosted_zone = result["HostedZone"]
        zone_id = hosted_zone["Id"].replace("/hostedzone/", "")
        record["zone_id"] = zone_id

        if len(record["vpcs"]) > 1:
            for vpc in record["vpcs"][1:]:
                try:
                    result = client.associate_vpc_with_hosted_zone(
                        HostedZoneId=zone_id,
                        VPC={
                            "VPCRegion": vpc["region"],
                            "VPCId": vpc["id"],
                        },
                    )
                except (BotoCoreError, ClientError) as e:
                    module.fail_json_aws(e, msg="Could not associate additional VPCs with hosted zone")

    changed = True
    return changed, record


def create_or_update_public(matching_zones, record):
    zone_details, zone_delegation_set_details = None, {}
    for matching_zone in matching_zones:
        try:
            zone = client.get_hosted_zone(Id=matching_zone["Id"])
            zone_details = zone["HostedZone"]
            zone_delegation_set_details = zone.get("DelegationSet", {})
        except (BotoCoreError, ClientError) as e:
            module.fail_json_aws(e, msg=f"Could not get details about hosted zone {matching_zone['Id']}")
        if "Comment" in zone_details["Config"] and zone_details["Config"]["Comment"] != record["comment"]:
            if not module.check_mode:
                try:
                    client.update_hosted_zone_comment(Id=zone_details["Id"], Comment=record["comment"])
                except (BotoCoreError, ClientError) as e:
                    module.fail_json_aws(e, msg=f"Could not update comment for hosted zone {zone_details['Id']}")
            changed = True
        else:
            changed = False
        break

    if zone_details is None:
        if not module.check_mode:
            try:
                params = dict(
                    Name=record["name"],
                    HostedZoneConfig={
                        "Comment": record["comment"] if record["comment"] is not None else "",
                        "PrivateZone": False,
                    },
                    CallerReference=f"{record['name']}-{time.time()}",
                )

                if record.get("delegation_set_id") is not None:
                    params["DelegationSetId"] = record["delegation_set_id"]

                result = client.create_hosted_zone(**params)
                zone_details = result["HostedZone"]
                zone_delegation_set_details = result.get("DelegationSet", {})

            except (BotoCoreError, ClientError) as e:
                module.fail_json_aws(e, msg="Could not create hosted zone")
        changed = True

    if module.check_mode:
        if zone_details:
            record["zone_id"] = zone_details["Id"].replace("/hostedzone/", "")
    else:
        record["zone_id"] = zone_details["Id"].replace("/hostedzone/", "")
        record["name"] = zone_details["Name"]
        record["delegation_set_id"] = zone_delegation_set_details.get("Id", "").replace("/delegationset/", "")

    return changed, record


def delete_private(matching_zones, vpcs):
    for z in matching_zones:
        try:
            result = client.get_hosted_zone(Id=z["Id"])
        except (BotoCoreError, ClientError) as e:
            module.fail_json_aws(e, msg=f"Could not get details about hosted zone {z['Id']}")
        zone_details = result["HostedZone"]
        vpc_details = result["VPCs"]
        if isinstance(vpc_details, dict):
            if vpc_details["VPC"]["VPCId"] == vpcs[0]["id"] and vpcs[0]["region"] == vpc_details["VPC"]["VPCRegion"]:
                if not module.check_mode:
                    try:
                        client.delete_hosted_zone(Id=z["Id"])
                    except (BotoCoreError, ClientError) as e:
                        module.fail_json_aws(e, msg=f"Could not delete hosted zone {z['Id']}")
                return True, f"Successfully deleted {zone_details['Name']}"
        else:
            # Sort the lists and compare them to make sure they contain the same items
            if sorted([vpc["id"] for vpc in vpcs]) == sorted([v["VPCId"] for v in vpc_details]) and sorted(
                [vpc["region"] for vpc in vpcs]
            ) == sorted([v["VPCRegion"] for v in vpc_details]):
                if not module.check_mode:
                    try:
                        client.delete_hosted_zone(Id=z["Id"])
                    except (BotoCoreError, ClientError) as e:
                        module.fail_json_aws(e, msg=f"Could not delete hosted zone {z['Id']}")
                return True, f"Successfully deleted {zone_details['Name']}"

    return False, "The VPCs do not match a private hosted zone."


def delete_public(matching_zones):
    if len(matching_zones) > 1:
        changed = False
        msg = "There are multiple zones that match. Use hosted_zone_id to specify the correct zone."
    else:
        if not module.check_mode:
            try:
                client.delete_hosted_zone(Id=matching_zones[0]["Id"])
            except (BotoCoreError, ClientError) as e:
                module.fail_json_aws(e, msg=f"Could not get delete hosted zone {matching_zones[0]['Id']}")
        changed = True
        msg = f"Successfully deleted {matching_zones[0]['Id']}"
    return changed, msg


def delete_hosted_id(hosted_zone_id, matching_zones):
    if hosted_zone_id == "all":
        deleted = []
        for z in matching_zones:
            deleted.append(z["Id"])
            if not module.check_mode:
                try:
                    client.delete_hosted_zone(Id=z["Id"])
                except (BotoCoreError, ClientError) as e:
                    module.fail_json_aws(e, msg=f"Could not delete hosted zone {z['Id']}")
        changed = True
        msg = f"Successfully deleted zones: {deleted}"
    elif hosted_zone_id in [zo["Id"].replace("/hostedzone/", "") for zo in matching_zones]:
        if not module.check_mode:
            try:
                client.delete_hosted_zone(Id=hosted_zone_id)
            except (BotoCoreError, ClientError) as e:
                module.fail_json_aws(e, msg=f"Could not delete hosted zone {hosted_zone_id}")
        changed = True
        msg = f"Successfully deleted zone: {hosted_zone_id}"
    else:
        changed = False
        msg = f"There is no zone to delete that matches hosted_zone_id {hosted_zone_id}."
    return changed, msg


def delete(matching_zones):
    zone_in = module.params.get("zone").lower()
    vpc_id = module.params.get("vpc_id")
    vpc_region = module.params.get("vpc_region")
    vpcs = module.params.get("vpcs") or ([{"id": vpc_id, "region": vpc_region}] if vpc_id and vpc_region else None)
    hosted_zone_id = module.params.get("hosted_zone_id")

    if not zone_in.endswith("."):
        zone_in += "."

    private_zone = bool(vpcs)

    if zone_in in [z["Name"] for z in matching_zones]:
        if hosted_zone_id:
            changed, result = delete_hosted_id(hosted_zone_id, matching_zones)
        else:
            if private_zone:
                changed, result = delete_private(matching_zones, vpcs)
            else:
                changed, result = delete_public(matching_zones)
    else:
        changed = False
        result = "No zone to delete."

    return changed, result


def main():
    global module
    global client

    argument_spec = dict(
        zone=dict(required=True),
        state=dict(default="present", choices=["present", "absent"]),
        vpc_id=dict(default=None),
        vpc_region=dict(default=None),
        vpcs=dict(
            type="list", default=None, elements="dict", options=dict(id=dict(required=True), region=dict(required=True))
        ),
        comment=dict(default=""),
        hosted_zone_id=dict(),
        delegation_set_id=dict(),
        tags=dict(type="dict", aliases=["resource_tags"]),
        purge_tags=dict(type="bool", default=True),
    )

    mutually_exclusive = [
        ["delegation_set_id", "vpc_id"],
        ["delegation_set_id", "vpc_region"],
        ["delegation_set_id", "vpcs"],
        ["vpcs", "vpc_id"],
        ["vpcs", "vpc_region"],
    ]

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        mutually_exclusive=mutually_exclusive,
        supports_check_mode=True,
    )

    zone_in = module.params.get("zone").lower()
    state = module.params.get("state").lower()
    vpc_id = module.params.get("vpc_id")
    vpc_region = module.params.get("vpc_region")
    vpcs = module.params.get("vpcs")

    if not zone_in.endswith("."):
        zone_in += "."

    private_zone = bool(vpcs or (vpc_id and vpc_region))

    client = module.client("route53", retry_decorator=AWSRetry.jittered_backoff())

    zones = find_zones(zone_in, private_zone)
    if state == "present":
        changed, result = create(matching_zones=zones)
    elif state == "absent":
        changed, result = delete(matching_zones=zones)

    if isinstance(result, dict):
        module.exit_json(changed=changed, result=result, **result)
    else:
        module.exit_json(changed=changed, result=result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


class ModuleDocFragment(object):
    # Standard files documentation fragment
    DOCUMENTATION = r"""
options:
  annotation:
    description:
    - User-defined string for annotating an object.
    - If the value is not specified in the task, the value of environment variable C(ACI_ANNOTATION) will be used instead.
    - If the value is not specified in the task and environment variable C(ACI_ANNOTATION) then the default value will be used.
    type: str
    default: orchestrator:ansible
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2022
# Marcin Woźniak (@y0rune) <y0rune@aol.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {
    "metadata_version": "1.1",
    "status": ["deprecated"],
    "supported_by": "community",
}

DOCUMENTATION = r"""
---
module: meraki_ms_access_policies
short_description: Manage Switch Access Policies in the Meraki cloud
description: Module for managing a Switch Access Policies in the Meraki cloud
deprecated:
  removed_in: '3.0.0'
  why: Updated modules released with increased functionality
  alternative: cisco.meraki.networks_switch_access_policies
options:
    state:
        description:
        - Specifies whether SNMP information should be queried or modified.
        type: str
        choices: [ absent, query, present ]
        default: present
    number:
        description:
        - Number of the access_policy.
        type: int
        aliases: [access_policy_number]
    name:
        description:
        - Name of Access Policy.
        type: str
    net_id:
        description:
        - ID of network.
        type: str
    org_id:
        description:
        - ID of organization associated to a network.
        type: str
    net_name:
        description:
        - Name of a network.
        aliases: [name, network]
        type: str
    auth_method:
        description:
        - Set authentication method in the policy.
        type: str
        choices: ["Meraki authentication", "my RADIUS server"]
    guest_vlan:
        description:
        - Guest Vlan
        type: int
    access_policy_type:
        description:
        - Set type of the access policy
        type: str
        choices: ["802.1x", "MAC authentication bypass", "Hybrid authentication"]
    systems_management_enrollment:
        description:
        - Set if the Systems Management Enrollemnt is enabled or disabled
        type: bool
        default: False
    radius_servers:
        description:
        - List of RADIUS servers.
        type: list
        elements: dict
        suboptions:
            host:
                description:
                - IP address or hostname of RADIUS server.
                type: str
                required: true
            port:
                description:
                - Port number RADIUS server is listening to.
                type: int
            secret:
                description:
                - RADIUS password.
                - Setting password is not idempotent.
                type: str
    radius_testing:
        description:
        - Set status of testing a radius.
        type: bool
        default: True
    voice_vlan_clients:
        description:
        - If is enabled that means Voice VLAN client require authentication
        type: bool
        default: True
    radius_coa_enabled:
        description:
        - Enable or disable RADIUS CoA (Change of Authorization).
        type: bool
    radius_accounting_enabled:
        description:
        - Enable or disable RADIUS accounting.
        type: bool
    radius_accounting_servers:
        description:
        - List of RADIUS servers for RADIUS accounting.
        type: list
        elements: dict
        suboptions:
            host:
                description:
                - IP address or hostname of RADIUS server.
                type: str
                required: true
            port:
                description:
                - Port number RADIUS server is listening to.
                type: int
            secret:
                description:
                - RADIUS password.
                type: str
    host_mode:
        description:
        - Choose the Host Mode for the access policy.
        type: str
        choices: ["Single-Host", "Multi-Domain", "Multi-Host", "Multi-Auth"]
    data_vlan_id:
        description:
        - Set a Data VLAN ID for Critical Auth VLAN
        type: int
    voice_vlan_id:
        description:
        - Set a Voice VLAN ID for Critical Auth VLAN
        type: int
    suspend_port_bounce:
        description:
        - Enable or disable the Suspend Port Bounce when RADIUS servers are unreachable.
        type: bool
        default: False
    radius_attribute_group_policy_name:
        description:
        - Enable that attribute for a RADIUS
        type: str
        choices: ["Filter-Id", ""]
        default: ""
author:
- Marcin Woźniak (@y0rune)
extends_documentation_fragment: cisco.meraki.meraki
"""

EXAMPLES = r"""
- name: Create access policy with auth_method is "Meraki authentication"
  cisco.meraki.meraki_ms_access_policies:
    auth_key: abc123
    state: present
    name: "Meraki authentication policy"
    auth_method: "Meraki authentication"
    net_name: YourNet
    org_name: YourOrg
  delegate_to: localhost

- name: Create access policy with auth_method is "my Radius Server"
  cisco.meraki.meraki_ms_access_policies:
    auth_key: abc123
    access_policy_type: "802.1x"
    host_mode: "Single-Host"
    state: present
    name: "Meraki authentication policy"
    auth_method: "my RADIUS server"
    radius_servers:
      - host: 192.0.1.18
        port: 7890
        secret: secret123
    net_name: YourNet
    org_name: YourOrg
    radius_coa_enabled: False
    radius_accounting_enabled: False
    guest_vlan: 10
    voice_vlan_clients: False
"""

RETURN = r"""
data:
    description: List of Access Policies
    returned: success
    type: complex
    contains:
        number:
            description: Number of the Access Policy
            returned: success
            type: int
            sample: 1
        name:
            description: Name of the Access Policy
            returned: success
            type: str
            sample: Policy with 802.1x
        access_policy_type:
            description: Type of the access policy
            returned: success
            type: str
            sample: 802.1x
        guest_vlan_id:
            description: ID of the Guest Vlan
            returned: success
            type: int
            sample: 10
        host_mode:
            description: Choosen teh Host Mode for the access policy
            returned: success
            type: str
            sample: Single-Host
        radius:
            description: List of radius specific list
            returned: success
            type: complex
            contains:
                critial_auth:
                    description: Critial Auth List
                    returned: success
                    type: complex
                    contains:
                        data_vlan_id:
                            description: VLAN ID for data
                            returned: success
                            type: int
                            sample: 10
                        suspend_port_bounce:
                            description: Enable or disable suspend port bounce
                            returned: success
                            type: bool
                            sample: False
                        voice_vlan_id:
                            description: VLAN ID for voice
                            returned: success
                            type: int
                            sample: 10
                failed_auth_vlan_id:
                    description: VLAN ID when failed auth
                    returned: success
                    type: int
                    sample: 11
                re_authentication_interval:
                    description: Interval of re-authentication
                    returned: success
                    type: int
                    sample:
        radius_coa_enabled:
            description:
            - Enable or disable RADIUS CoA (Change of Authorization).
            type: bool
        radius_accounting_enabled:
            description:
            - Enable or disable RADIUS accounting.
            type: bool
        radius_accounting_servers:
            description:
            - List of RADIUS servers for RADIUS accounting.
            type: list
            elements: dict
        radius_servers:
            description:
            - List of RADIUS servers.
            type: list
            elements: dict
        radius_attribute_group_policy_name:
            description: Enable the radius group attribute
            returned: success
            type: str
            choices: [ "11", ""]
            sample: 11
        radius_testing_enabled:
            description: Enable or disable Radius Testing
            returned: success
            type: bool
            sample: True
        voice_vlan_clients:
            description: Enable or disable Voice Vlan Clients
            returned: success
            type: bool
            sample: False
"""

from ansible.module_utils.basic import AnsibleModule, json
from ansible_collections.cisco.meraki.plugins.module_utils.network.meraki.meraki import (
    MerakiModule,
    meraki_argument_spec,
)


def convert_vlan_id(vlan_id):
    if vlan_id == "":
        return None
    elif vlan_id == 0:
        return None
    elif vlan_id in range(1, 4094):
        return vlan_id


def convert_radius_attribute_group_policy_name(arg):
    if arg == "Filter-Id":
        return 11
    else:
        return ""


def main():
    argument_spec = meraki_argument_spec()

    radius_arg_spec = dict(
        host=dict(type="str", required=True),
        port=dict(type="int"),
        secret=dict(type="str", no_log=True),
    )

    argument_spec.update(
        state=dict(
            type="str",
            choices=["present", "query", "absent"],
            default="present",
        ),
        net_id=dict(type="str"),
        net_name=dict(type="str", aliases=["network"]),
        number=dict(type="int", aliases=["access_policy_number"]),
        name=dict(type="str"),
        auth_method=dict(
            type="str",
            choices=["Meraki authentication", "my RADIUS server"],
        ),
        guest_vlan=dict(type="int"),
        access_policy_type=dict(
            type="str",
            choices=[
                "802.1x",
                "MAC authentication bypass",
                "Hybrid authentication",
            ],
        ),
        systems_management_enrollment=dict(type="bool", default=False),
        radius_servers=dict(
            type="list", default=None, elements="dict", options=radius_arg_spec
        ),
        radius_testing=dict(type="bool", default="True"),
        voice_vlan_clients=dict(type="bool", default="True"),
        radius_coa_enabled=dict(type="bool"),
        radius_accounting_enabled=dict(type="bool"),
        radius_accounting_servers=dict(
            type="list", elements="dict", options=radius_arg_spec
        ),
        host_mode=dict(
            type="str",
            choices=["Single-Host", "Multi-Domain", "Multi-Host", "Multi-Auth"],
        ),
        data_vlan_id=dict(type="int"),
        voice_vlan_id=dict(type="int"),
        suspend_port_bounce=dict(type="bool", default="False"),
        radius_attribute_group_policy_name=dict(
            type="str", choices=["Filter-Id", ""], default=""
        ),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )
    meraki = MerakiModule(module, function="access_policies")

    net_id = meraki.params["net_id"]
    net_name = meraki.params["net_name"]

    org_id = meraki.params["org_id"]
    org_name = meraki.params["org_name"]

    if meraki.params["net_name"] and meraki.params["net_id"]:
        meraki.fail_json(msg="net_name and net_id are mutually exclusive")

    if meraki.params["org_name"] and meraki.params["org_id"]:
        meraki.fail_json(msg="org_name and org_id are mutually exclusive")

    if net_id or net_name:
        if net_id is None:
            if org_id is None:
                org_id = meraki.get_org_id(org_name)
            nets = meraki.get_nets(org_id=org_id)
            net_id = meraki.get_net_id(net_name=net_name, data=nets)

    query_urls = {"access_policies": "/networks/{net_id}/switch/accessPolicies"}
    query_url = {
        "access_policies": "/networks/{net_id}/switch/accessPolicies/{number}"
    }
    update_url = {
        "access_policies": "/networks/{net_id}/switch/accessPolicies/{number}"
    }
    create_url = {"access_policies": "/networks/{net_id}/switch/accessPolicies"}

    meraki.url_catalog["get_all"].update(query_urls)
    meraki.url_catalog["get_one"].update(query_url)
    meraki.url_catalog["update"] = update_url
    meraki.url_catalog["create"] = create_url

    payload_auth = {
        "name": meraki.params["name"],
        "radiusServers": [],
        "radiusTestingEnabled": False,
        "radiusGroupAttribute": meraki.params[
            "radius_attribute_group_policy_name"
        ],
        "radius": {
            "criticalAuth": {
                "dataVlanId": None,
                "voiceVlanId": None,
                "suspendPortBounce": False,
            },
            "failedAuthVlanId": None,
            "reAuthenticationInterval": None,
        },
        "radiusCoaSupportEnabled": False,
        "radiusAccountingEnabled": False,
        "radiusAccountingServers": [],
        "hostMode": "Single-Host",
        "accessPolicyType": "802.1x",
        "voiceVlanClients": True,
        "systems_management_enrollment": meraki.params[
            "systems_management_enrollment"
        ],
        "guestVlanId": meraki.params["guest_vlan"],
        "urlRedirectWalledGardenEnabled": False,
    }

    payload_radius = {
        "name": meraki.params["name"],
        "radiusServers": meraki.params["radius_servers"],
        "radiusTestingEnabled": meraki.params["radius_testing"],
        "radiusGroupAttribute": convert_radius_attribute_group_policy_name(
            meraki.params["radius_attribute_group_policy_name"]
        ),
        "radius": {
            "criticalAuth": {
                "dataVlanId": convert_vlan_id(meraki.params["data_vlan_id"]),
                "voiceVlanId": convert_vlan_id(meraki.params["voice_vlan_id"]),
                "suspendPortBounce": meraki.params["suspend_port_bounce"],
            },
            "failedAuthVlanId": None,
            "reAuthenticationInterval": None,
        },
        "radiusCoaSupportEnabled": meraki.params["radius_coa_enabled"],
        "hostMode": meraki.params["host_mode"],
        "accessPolicyType": meraki.params["access_policy_type"],
        "guestVlanId": meraki.params["guest_vlan"],
        "voiceVlanClients": meraki.params["voice_vlan_clients"],
        "urlRedirectWalledGardenEnabled": False,
        "radiusAccountingEnabled": meraki.params["radius_accounting_enabled"],
        "radiusAccountingServers": meraki.params["radius_accounting_servers"],
        "systems_management_enrollment": meraki.params[
            "systems_management_enrollment"
        ],
    }

    if meraki.params["state"] == "query":
        if meraki.params["number"]:
            path = meraki.construct_path(
                "get_one",
                net_id=net_id,
                custom={
                    "number": meraki.params["number"],
                },
            )
            response = meraki.request(path, method="GET")
            meraki.result["data"] = response
        else:
            path = meraki.construct_path("get_all", net_id=net_id)
            response = meraki.request(path, method="GET")
            meraki.result["data"] = response
    elif meraki.params["state"] == "present":

        query_path_all = meraki.construct_path(
            "get_all",
            net_id=net_id,
        )

        original_all = meraki.request(query_path_all, method="GET")

        for i in original_all:
            if i.get("name") == meraki.params["name"]:
                meraki.params["number"] = i.get("accessPolicyNumber")

        if meraki.params["number"] is None:
            path = meraki.construct_path(
                "create",
                net_id=net_id,
            )
            if meraki.params["auth_method"] == "Meraki authentication":
                response = meraki.request(
                    path, method="POST", payload=json.dumps(payload_auth)
                )
                meraki.result["changed"] = True
                meraki.result["data"] = response
            elif meraki.params["auth_method"] == "my RADIUS server":
                response = meraki.request(
                    path, method="POST", payload=json.dumps(payload_radius)
                )
                meraki.result["changed"] = True
                meraki.result["data"] = response
        else:
            query_path = meraki.construct_path(
                "get_one",
                net_id=net_id,
                custom={
                    "number": meraki.params["number"],
                },
            )

            update_path = meraki.construct_path(
                "update",
                net_id=net_id,
                custom={
                    "number": meraki.params["number"],
                },
            )

            proposed = ""

            if meraki.params["auth_method"] == "Meraki authentication":
                proposed = payload_auth.copy()
            elif meraki.params["auth_method"] == "my RADIUS server":
                proposed = payload_radius.copy()

            original = meraki.request(query_path, method="GET")

            ignored_parameters = [
                "accessPolicyNumber",
                "secret",
                "systems_management_enrollment",
            ]

            if meraki.params["radius_accounting_enabled"]:
                proposed.update(
                    {
                        "radiusAccountingServers": meraki.params[
                            "radius_accounting_servers"
                        ],
                    }
                )
            else:
                proposed.update(
                    {
                        "radiusAccountingServers": [],
                    }
                )

            if meraki.params["radius_servers"]:
                proposed.update(
                    {
                        "radiusServers": meraki.params["radius_servers"],
                    }
                )
            else:
                proposed.update(
                    {
                        "radiusServers": [],
                    }
                )

            if meraki.is_update_required(
                original,
                proposed,
                optional_ignore=ignored_parameters,
            ):

                if meraki.check_mode is True:
                    original.update(proposed)
                    meraki.result["data"] = original
                    meraki.result["changed"] = True
                    meraki.exit_json(**meraki.result)

                response = meraki.request(
                    update_path, method="PUT", payload=json.dumps(proposed)
                )
                meraki.result["changed"] = True
                meraki.result["data"] = response
            else:
                meraki.result["data"] = original

    elif meraki.params["state"] == "absent":
        path = meraki.construct_path(
            "update",
            net_id=net_id,
            custom={
                "number": meraki.params["number"],
            },
        )

        response = meraki.request(path, method="DELETE")
        meraki.result["changed"] = True
        meraki.result["data"] = response

    meraki.exit_json(**meraki.result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: transit_peer_network
short_description: Resource module for Transit Peer Network
description:
- Manage operations create and delete of the resource Transit Peer Network.
- Add Transit Peer Network in SD-Access.
- Delete Transit Peer Network from SD-Access.
version_added: '6.5.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  ipTransitSettings:
    description: Transit Peer Network's ipTransitSettings.
    suboptions:
      autonomousSystemNumber:
        description: Autonomous System Number (e.g.,1-65535).
        type: str
      routingProtocolName:
        description: Routing Protocol Name.
        type: str
    type: dict
  sdaTransitSettings:
    description: Transit Peer Network's sdaTransitSettings.
    suboptions:
      transitControlPlaneSettings:
        description: Transit Peer Network's transitControlPlaneSettings.
        elements: dict
        suboptions:
          deviceManagementIpAddress:
            description: Device Management Ip Address of provisioned device.
            type: str
          siteNameHierarchy:
            description: Site Name Hierarchy where device is provisioned.
            type: str
        type: list
    type: dict
  transitPeerNetworkName:
    description: TransitPeerNetworkName query parameter. Transit Peer Network Name.
    type: str
  transitPeerNetworkType:
    description: Transit Peer Network Type.
    type: str
requirements:
- dnacentersdk >= 2.5.5
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for  AddTransitPeerNetwork
  description: Complete reference of the AddTransitPeerNetwork API.
  link: https://developer.cisco.com/docs/dna-center/#!add-transit-peer-network
- name: Cisco DNA Center documentation for  DeleteTransitPeerNetwork
  description: Complete reference of the DeleteTransitPeerNetwork API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-transit-peer-network
notes:
  - SDK Method used are
    ..add_transit_peer_network,
    ..delete_transit_peer_network,

  - Paths used are
    post /dna/intent/api/v1/business/sda/transit-peer-network,
    delete /dna/intent/api/v1/business/sda/transit-peer-network,

"""

EXAMPLES = r"""
- name: Delete all
  cisco.dnac.transit_peer_network:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    transitPeerNetworkName: string

- name: Create
  cisco.dnac.transit_peer_network:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    ipTransitSettings:
      autonomousSystemNumber: string
      routingProtocolName: string
    sdaTransitSettings:
      transitControlPlaneSettings:
      - deviceManagementIpAddress: string
        siteNameHierarchy: string
    transitPeerNetworkName: string
    transitPeerNetworkType: string

"""

RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "status": "string",
      "description": "string",
      "taskId": "string",
      "taskStatusUrl": "string",
      "executionStatusUrl": "string",
      "executionId": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: tasks_info
short_description: Information module for Tasks
description:
- Get all Tasks.
- Get Tasks by id.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options:
  taskId:
    description:
    - TaskId path parameter. The id of the task executed before.
    type: str
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
seealso:
- name: Cisco ISE documentation for tasks
  description: Complete reference of the tasks API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!task-service-openapi
notes:
  - SDK Method used are
    tasks.Tasks.get_task_status,
    tasks.Tasks.get_task_status_by_id,

  - Paths used are
    get /api/v1/task,
    get /api/v1/task/{taskId},

"""

EXAMPLES = r"""
- name: Get all Tasks
  cisco.ise.tasks_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
  register: result

- name: Get Tasks by id
  cisco.ise.tasks_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    taskId: string
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "detailStatus": [
        {}
      ],
      "executionStatus": "string",
      "failCount": 0,
      "id": "string",
      "moduleType": "string",
      "resourcesCount": 0,
      "startTime": "string",
      "successCount": 0
    }
"""



# Copyright (c) 2023, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


import abc
import json
import shlex

from ansible.module_utils.basic import AnsibleModule, env_fallback
from ansible.module_utils.common.process import get_bin_path
from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.docker.plugins.module_utils.version import LooseVersion

from ansible_collections.community.docker.plugins.module_utils._api.auth import resolve_repository_name

from ansible_collections.community.docker.plugins.module_utils.util import (  # noqa: F401, pylint: disable=unused-import
    DEFAULT_DOCKER_HOST,
    DEFAULT_TLS,
    DEFAULT_TLS_VERIFY,
    DOCKER_MUTUALLY_EXCLUSIVE,
    DOCKER_REQUIRED_TOGETHER,
    sanitize_result,
)


DOCKER_COMMON_ARGS = dict(
    docker_cli=dict(type='path'),
    docker_host=dict(type='str', default=DEFAULT_DOCKER_HOST, fallback=(env_fallback, ['DOCKER_HOST']), aliases=['docker_url']),
    tls_hostname=dict(type='str', fallback=(env_fallback, ['DOCKER_TLS_HOSTNAME'])),
    api_version=dict(type='str', default='auto', fallback=(env_fallback, ['DOCKER_API_VERSION']), aliases=['docker_api_version']),
    ca_path=dict(type='path', aliases=['ca_cert', 'tls_ca_cert', 'cacert_path']),
    client_cert=dict(type='path', aliases=['tls_client_cert', 'cert_path']),
    client_key=dict(type='path', aliases=['tls_client_key', 'key_path']),
    tls=dict(type='bool', default=DEFAULT_TLS, fallback=(env_fallback, ['DOCKER_TLS'])),
    validate_certs=dict(type='bool', default=DEFAULT_TLS_VERIFY, fallback=(env_fallback, ['DOCKER_TLS_VERIFY']), aliases=['tls_verify']),
    # debug=dict(type='bool', default=False),
    cli_context=dict(type='str'),
)


class DockerException(Exception):
    pass


class AnsibleDockerClientBase(object):
    def __init__(self, common_args, min_docker_api_version=None):
        self._environment = {}
        if common_args['tls_hostname']:
            self._environment['DOCKER_TLS_HOSTNAME'] = common_args['tls_hostname']
        if common_args['api_version'] and common_args['api_version'] != 'auto':
            self._environment['DOCKER_API_VERSION'] = common_args['api_version']
        self._cli = common_args.get('docker_cli')
        if self._cli is None:
            try:
                self._cli = get_bin_path('docker')
            except ValueError:
                self.fail('Cannot find docker CLI in path. Please provide it explicitly with the docker_cli parameter')

        self._cli_base = [self._cli]
        self._cli_base.extend(['--host', common_args['docker_host']])
        if common_args['validate_certs']:
            self._cli_base.append('--tlsverify')
        elif common_args['tls']:
            self._cli_base.append('--tls')
        if common_args['ca_path']:
            self._cli_base.extend(['--tlscacert', common_args['ca_path']])
        if common_args['client_cert']:
            self._cli_base.extend(['--tlscert', common_args['client_cert']])
        if common_args['client_key']:
            self._cli_base.extend(['--tlskey', common_args['client_key']])
        if common_args['cli_context']:
            self._cli_base.extend(['--context', common_args['cli_context']])

        # `--format json` was only added as a shorthand for `--format {{ json . }}` in Docker 23.0
        dummy, self._version, dummy = self.call_cli_json('version', '--format', '{{ json . }}', check_rc=True)
        self._info = None

        self.docker_api_version_str = self._version['Server']['ApiVersion']
        self.docker_api_version = LooseVersion(self.docker_api_version_str)
        min_docker_api_version = min_docker_api_version or '1.25'
        if self.docker_api_version < LooseVersion(min_docker_api_version):
            self.fail('Docker API version is %s. Minimum version required is %s.' % (self.docker_api_version_str, min_docker_api_version))

    def log(self, msg, pretty_print=False):
        pass
        # if self.debug:
        #     from .util import log_debug
        #     log_debug(msg, pretty_print=pretty_print)

    def get_cli(self):
        return self._cli

    def get_version_info(self):
        return self._version

    def _compose_cmd(self, args):
        return self._cli_base + list(args)

    def _compose_cmd_str(self, args):
        return ' '.join(shlex.quote(a) for a in self._compose_cmd(args))

    @abc.abstractmethod
    # def call_cli(self, *args, check_rc=False, data=None, cwd=None, environ_update=None):
    def call_cli(self, *args, **kwargs):
        # Python 2.7 doesn't like anything than '**kwargs' after '*args', so we have to do this manually...
        pass

    # def call_cli_json(self, *args, check_rc=False, data=None, cwd=None, environ_update=None, warn_on_stderr=False):
    def call_cli_json(self, *args, **kwargs):
        warn_on_stderr = kwargs.pop('warn_on_stderr', False)
        rc, stdout, stderr = self.call_cli(*args, **kwargs)
        if warn_on_stderr and stderr:
            self.warn(to_native(stderr))
        try:
            data = json.loads(stdout)
        except Exception as exc:
            self.fail('Error while parsing JSON output of {cmd}: {exc}\nJSON output: {stdout}'.format(
                cmd=self._compose_cmd_str(args),
                exc=to_native(exc),
                stdout=to_native(stdout),
            ))
        return rc, data, stderr

    # def call_cli_json_stream(self, *args, check_rc=False, data=None, cwd=None, environ_update=None, warn_on_stderr=False):
    def call_cli_json_stream(self, *args, **kwargs):
        warn_on_stderr = kwargs.pop('warn_on_stderr', False)
        rc, stdout, stderr = self.call_cli(*args, **kwargs)
        if warn_on_stderr and stderr:
            self.warn(to_native(stderr))
        result = []
        try:
            for line in stdout.splitlines():
                line = line.strip()
                if line.startswith(b'{'):
                    result.append(json.loads(line))
        except Exception as exc:
            self.fail('Error while parsing JSON output of {cmd}: {exc}\nJSON output: {stdout}'.format(
                cmd=self._compose_cmd_str(args),
                exc=to_native(exc),
                stdout=to_native(stdout),
            ))
        return rc, result, stderr

    @abc.abstractmethod
    def fail(self, msg, **kwargs):
        pass

    @abc.abstractmethod
    def warn(self, msg):
        pass

    @abc.abstractmethod
    def deprecate(self, msg, version=None, date=None, collection_name=None):
        pass

    def get_cli_info(self):
        if self._info is None:
            dummy, self._info, dummy = self.call_cli_json('info', '--format', '{{ json . }}', check_rc=True)
        return self._info

    def get_client_plugin_info(self, component):
        for plugin in self.get_cli_info()['ClientInfo'].get('Plugins') or []:
            if plugin.get('Name') == component:
                return plugin
        return None

    def _image_lookup(self, name, tag):
        '''
        Including a tag in the name parameter sent to the Docker SDK for Python images method
        does not work consistently. Instead, get the result set for name and manually check
        if the tag exists.
        '''
        dummy, images, dummy = self.call_cli_json_stream(
            'image', 'ls', '--format', '{{ json . }}', '--no-trunc', '--filter', 'reference={0}'.format(name),
            check_rc=True,
        )
        if tag:
            lookup = "%s:%s" % (name, tag)
            lookup_digest = "%s@%s" % (name, tag)
            response = images
            images = []
            for image in response:
                if image.get('Tag') == tag or image.get('Digest') == tag:
                    images = [image]
                    break
        return images

    def find_image(self, name, tag):
        '''
        Lookup an image (by name and tag) and return the inspection results.
        '''
        if not name:
            return None

        self.log("Find image %s:%s" % (name, tag))
        images = self._image_lookup(name, tag)
        if not images:
            # In API <= 1.20 seeing 'docker.io/<name>' as the name of images pulled from docker hub
            registry, repo_name = resolve_repository_name(name)
            if registry == 'docker.io':
                # If docker.io is explicitly there in name, the image
                # isn't found in some cases (#41509)
                self.log("Check for docker.io image: %s" % repo_name)
                images = self._image_lookup(repo_name, tag)
                if not images and repo_name.startswith('library/'):
                    # Sometimes library/xxx images are not found
                    lookup = repo_name[len('library/'):]
                    self.log("Check for docker.io image: %s" % lookup)
                    images = self._image_lookup(lookup, tag)
                if not images:
                    # Last case for some Docker versions: if docker.io wasn't there,
                    # it can be that the image wasn't found either
                    # (https://github.com/ansible/ansible/pull/15586)
                    lookup = "%s/%s" % (registry, repo_name)
                    self.log("Check for docker.io image: %s" % lookup)
                    images = self._image_lookup(lookup, tag)
                if not images and '/' not in repo_name:
                    # This seems to be happening with podman-docker
                    # (https://github.com/ansible-collections/community.docker/issues/291)
                    lookup = "%s/library/%s" % (registry, repo_name)
                    self.log("Check for docker.io image: %s" % lookup)
                    images = self._image_lookup(lookup, tag)

        if len(images) > 1:
            self.fail("Daemon returned more than one result for %s:%s" % (name, tag))

        if len(images) == 1:
            rc, image, stderr = self.call_cli_json('image', 'inspect', images[0]['ID'])
            if not image:
                self.log("Image %s:%s not found." % (name, tag))
                return None
            if rc != 0:
                self.fail("Error inspecting image %s:%s - %s" % (name, tag, to_native(stderr)))
            return image[0]

        self.log("Image %s:%s not found." % (name, tag))
        return None

    def find_image_by_id(self, image_id, accept_missing_image=False):
        '''
        Lookup an image (by ID) and return the inspection results.
        '''
        if not image_id:
            return None

        self.log("Find image %s (by ID)" % image_id)
        rc, image, stderr = self.call_cli_json('image', 'inspect', image_id)
        if not image:
            if not accept_missing_image:
                self.fail("Error inspecting image ID %s - %s" % (image_id, to_native(stderr)))
            self.log("Image %s not found." % image_id)
            return None
        if rc != 0:
            self.fail("Error inspecting image ID %s - %s" % (image_id, to_native(stderr)))
        return image[0]


class AnsibleModuleDockerClient(AnsibleDockerClientBase):
    def __init__(self, argument_spec=None, supports_check_mode=False, mutually_exclusive=None,
                 required_together=None, required_if=None, required_one_of=None, required_by=None,
                 min_docker_api_version=None, fail_results=None):

        # Modules can put information in here which will always be returned
        # in case client.fail() is called.
        self.fail_results = fail_results or {}

        merged_arg_spec = dict()
        merged_arg_spec.update(DOCKER_COMMON_ARGS)
        if argument_spec:
            merged_arg_spec.update(argument_spec)
            self.arg_spec = merged_arg_spec

        mutually_exclusive_params = []
        mutually_exclusive_params += DOCKER_MUTUALLY_EXCLUSIVE
        if mutually_exclusive:
            mutually_exclusive_params += mutually_exclusive

        required_together_params = []
        required_together_params += DOCKER_REQUIRED_TOGETHER
        if required_together:
            required_together_params += required_together

        self.module = AnsibleModule(
            argument_spec=merged_arg_spec,
            supports_check_mode=supports_check_mode,
            mutually_exclusive=mutually_exclusive_params,
            required_together=required_together_params,
            required_if=required_if,
            required_one_of=required_one_of,
            required_by=required_by or {},
        )

        self.debug = False  # self.module.params['debug']
        self.check_mode = self.module.check_mode
        self.diff = self.module._diff

        common_args = dict((k, self.module.params[k]) for k in DOCKER_COMMON_ARGS)
        super(AnsibleModuleDockerClient, self).__init__(common_args, min_docker_api_version=min_docker_api_version)

    # def call_cli(self, *args, check_rc=False, data=None, cwd=None, environ_update=None):
    def call_cli(self, *args, **kwargs):
        # Python 2.7 doesn't like anything than '**kwargs' after '*args', so we have to do this manually...
        check_rc = kwargs.pop('check_rc', False)
        data = kwargs.pop('data', None)
        cwd = kwargs.pop('cwd', None)
        environ_update = kwargs.pop('environ_update', None)
        if kwargs:
            raise TypeError("call_cli() got an unexpected keyword argument '%s'" % list(kwargs)[0])

        environment = self._environment.copy()
        if environ_update:
            environment.update(environ_update)
        rc, stdout, stderr = self.module.run_command(
            self._compose_cmd(args),
            binary_data=True,
            check_rc=check_rc,
            cwd=cwd,
            data=data,
            encoding=None,
            environ_update=environment,
            expand_user_and_vars=False,
            ignore_invalid_cwd=False,
        )
        return rc, stdout, stderr

    def fail(self, msg, **kwargs):
        self.fail_results.update(kwargs)
        self.module.fail_json(msg=msg, **sanitize_result(self.fail_results))

    def warn(self, msg):
        self.module.warn(msg)

    def deprecate(self, msg, version=None, date=None, collection_name=None):
        self.module.deprecate(msg, version=version, date=date, collection_name=collection_name)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: licensing_registration_create
short_description: Resource module for Licensing Registration Create
description:
- Manage operation create of the resource Licensing Registration Create.
- License - Configure registration information.
version_added: '2.1.0'
extends_documentation_fragment:
  - cisco.ise.module
author: Rafael Campos (@racampos)
options:
  connectionType:
    description: Licensing Registration Create's connectionType.
    type: str
  registrationType:
    description: Licensing Registration Create's registrationType.
    type: str
  ssmOnPremServer:
    description: If connection type is selected as SSM_ONPREM_SERVER, then IP address
      or the hostname (or FQDN) of the SSM On-Prem server Host.
    type: str
  tier:
    description: Licensing Registration Create's tier.
    elements: str
    type: list
  token:
    description: Token.
    type: str
requirements:
- ciscoisesdk >= 2.1.1
- python >= 3.5
seealso:
- name: Cisco ISE documentation for Licensing
  description: Complete reference of the Licensing API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!license-openapi
notes:
  - SDK Method used are
    licensing.Licensing.create_registration_info,

  - Paths used are
    post /api/v1/license/system/register,

"""

EXAMPLES = r"""
- name: Create
  cisco.ise.licensing_registration_create:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
    connectionType: string
    registrationType: string
    ssmOnPremServer: string
    tier:
    - string
    token: string

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: str
  sample: >
    "'string'"
"""



# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Bgp_templates parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def _tmplt_bfd(proc):
    bfd = proc.get("bfd", {})
    cmd = None

    if bfd.get("set"):
        cmd = "bfd"
    if bfd.get("singlehop"):
        cmd = "bfd singlehop"
    elif bfd.get("multihop", {}).get("set"):
        cmd = "bfd multihop"

    return cmd


def _tmplt_path_attribute(proc):
    cmd = "path-attribute {action}".format(**proc)

    if "type" in proc:
        cmd += " {type}".format(**proc)
    elif "range" in proc:
        cmd += " range {start} {end}".format(**proc["range"])
    cmd += " in"
    return cmd


class Bgp_templatesTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Bgp_templatesTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "as_number",
            "getval": re.compile(
                r"""
                ^router\sbgp\s(?P<as_number>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "router bgp {{ as_number }}",
            "result": {
                "as_number": "{{ as_number }}",
            },
        },
        {
            "name": "peer.name",
            "getval": re.compile(
                r"""
                template\speer\s(?P<name>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "template peer {{ name }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "name": "{{ name }}",
                    },
                },
            },
            "shared": True,
        },
        {
            "name": "bfd",
            "getval": re.compile(
                r"""
                (?P<bfd>bfd)
                (\s(?P<singlehop>singlehop))?
                (\s(?P<multihop>multihop))?
                $""", re.VERBOSE,
            ),
            "setval": _tmplt_bfd,
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "bfd": {
                            "set": "{{ True if bfd is defined and singlehop is undefined and multihop is undefined else None }}",
                            "singlehop": "{{ not not singlehop }}",
                            "multihop": {
                                "set": "{{ not not multihop }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bfd.multihop.interval",
            "getval": re.compile(
                r"""
                bfd\smultihop\sinterval
                \s(?P<tx_interval>\d+)
                \smin_rx\s(?P<min_rx_interval>\d+)
                \smultiplier\s(?P<multiplier>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "bfd multihop interval"
                      " {{ bfd.multihop.interval.tx_interval }}"
                      " min_rx {{ bfd.multihop.interval.min_rx_interval }}"
                      " multiplier {{ bfd.multihop.interval.multiplier }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "bfd": {
                            "multihop": {
                                "interval": {
                                    "tx_interval": "{{ tx_interval }}",
                                    "min_rx_interval": "{{ min_rx_interval }}",
                                    "multiplier": "{{ multiplier }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bmp_activate_server",
            "getval": re.compile(
                r"""
                bmp-activate-server\s(?P<bmp_activate_server>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "bmp-activate-server {{ bmp_activate_server }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "bmp_activate_server": "{{ bmp_activate_server }}",
                    },
                },
            },
        },
        {
            "name": "capability",
            "getval": re.compile(
                r"""
                capability\ssuppress\s(?P<suppress_4_byte_as>4-byte-as)
                $""", re.VERBOSE,
            ),
            "setval": "capability suppress 4-byte-as",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "capability": {
                            "suppress_4_byte_as": "{{ not not suppress_4_byte_as }}",
                        },
                    },
                },
            },
        },
        {
            "name": "description",
            "getval": re.compile(
                r"""
                description\s(?P<description>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "description {{ description }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "description": "{{ description }}",
                    },
                },
            },
        },
        {
            "name": "disable_connected_check",
            "getval": re.compile(
                r"""
                (?P<disable_connected_check>disable-connected-check)
                $""", re.VERBOSE,
            ),
            "setval": "disable-connected-check",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "disable_connected_check": "{{ not not disable_connected_check }}",
                    },
                },
            },
        },
        {
            "name": "dont_capability_negotiate",
            "getval": re.compile(
                r"""
                (?P<dont_capability_negotiate>dont-capability-negotiate)
                $""", re.VERBOSE,
            ),
            "setval": "dont-capability-negotiate",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "dont_capability_negotiate": "{{ not not dont_capability_negotiate }}",
                    },
                },
            },
        },
        {
            "name": "dscp",
            "getval": re.compile(
                r"""
                dscp\s(?P<dscp>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "dscp {{ dscp }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "dscp": "{{ dscp }}",
                    },
                },
            },
        },
        {
            "name": "dynamic_capability",
            "getval": re.compile(
                r"""
                (?P<dynamic_capability>dynamic-capability)
                $""", re.VERBOSE,
            ),
            "setval": "dynamic-capability",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "dynamic_capability": "{{ not not dynamic_capability }}",
                    },
                },
            },
        },
        {
            "name": "ebgp_multihop",
            "getval": re.compile(
                r"""
                ebgp-multihop\s(?P<ebgp_multihop>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "ebgp-multihop {{ ebgp_multihop }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "ebgp_multihop": "{{ ebgp_multihop }}",
                    },
                },
            },
        },
        {
            "name": "graceful_shutdown",
            "getval": re.compile(
                r"""
                graceful-shutdown
                \s(?P<activate>activate)
                (\sroute-map\s(?P<route_map>\S+))?
                $""", re.VERBOSE,
            ),
            "setval": "graceful-shutdown activate"
                      "{{ (' route-map ' + graceful_shutdown.activate.route_map) if graceful_shutdown.activate.route_map is defined }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "graceful_shutdown": {
                            "activate": {
                                "set": "{{ True if activate is defined and route_map is undefined else None }}",
                                "route_map": "{{ route_map }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "inherit.peer_session",
            "getval": re.compile(
                r"""
                inherit
                \speer-session\s(?P<peer_session>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "inherit peer-session {{ inherit.peer_session }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "inherit": {
                            "peer_session": "{{ peer_session }}",
                        },
                    },
                },
            },
        },
        {
            "name": "local_as",
            "getval": re.compile(
                r"""
                local-as\s(?P<local_as>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "local-as {{ local_as }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "local_as": "{{ local_as }}",
                    },
                },
            },
        },
        {
            "name": "log_neighbor_changes",
            "getval": re.compile(
                r"""
                (?P<log_neighbor_changes>log-neighbor-changes)
                (\s(?P<disable>disable))?
                $""", re.VERBOSE,
            ),
            "setval": "log-neighbor-changes{{ ' disable' if log_neighbor_changes.disable is defined }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "log_neighbor_changes": {
                            "set": "{{ True if log_neighbor_changes is defined and disable is undefined }}",
                            "disable": "{{ not not disable }}",
                        },
                    },
                },
            },
        },
        {
            "name": "low_memory",
            "getval": re.compile(
                r"""
                low-memory\s(?P<exempt>exempt)
                $""", re.VERBOSE,
            ),
            "setval": "low-memory exempt",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "low_memory": {
                            "exempt": "{{ not not exempt }}",
                        },
                    },
                },
            },
        },
        {
            "name": "password",
            "getval": re.compile(
                r"""
                password\s(?P<encryption>\d+)\s(?P<key>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "password{{ (' ' + password.encryption|string) if password.encryption is defined }} {{ password.key }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "password": {
                            "encryption": "{{ encryption }}",
                            "key": "{{ key }}",
                        },
                    },
                },
            },
        },
        {
            "name": "path_attribute",
            "getval": re.compile(
                r"""
                path-attribute\s(?P<action>\S+)\s
                (?P<type>\d+)?
                (range\s(?P<start>\d+)\s(?P<end>\d+))?
                \sin
                $""", re.VERBOSE,
            ),
            "setval": _tmplt_path_attribute,
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "path_attribute": [
                            {
                                "action": "{{ action }}",
                                "type": "{{ type if type is defined else None }}",
                                "range": {
                                    "start": "{{ start if start is defined }}",
                                    "end": "{{ end if end is defined }}",
                                },
                            },
                        ],
                    },
                },
            },
        },
        {
            "name": "remote_as",
            "getval": re.compile(
                r"""
                remote-as\s(?P<remote_as>.+)
                $""", re.VERBOSE,
            ),
            "setval": "remote-as {{ remote_as }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "remote_as": "{{ remote_as|string }}",
                    },
                },
            },
        },
        {
            "name": "remove_private_as",
            "getval": re.compile(
                r"""
                (?P<remove_private_as>remove-private-as)
                (\s(?P<all>all))?
                (\s(?P<replace_as>replace-as))?
                $""", re.VERBOSE,
            ),
            "setval": "remove-private-as"
                      "{{ ' replace-as' if remove_private_as.replace_as|d(False) else '' }}"
                      "{{ ' all' if remove_private_as.all|d(False) else '' }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "remove_private_as": {
                            "set": "{{ True if remove_private_as is defined and replace_as is undefined and all is undefined else None }}",
                            "replace_as": "{{ not not replace_as }}",
                            "all": "{{ not not all }}",
                        },
                    },
                },
            },
        },
        {
            "name": "shutdown",
            "getval": re.compile(
                r"""
                (?P<shutdown>shutdown)
                $""", re.VERBOSE,
            ),
            "setval": "shutdown",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "shutdown": "{{ not not shutdown }}",
                    },
                },
            },
        },
        {
            "name": "timers",
            "getval": re.compile(
                r"""
                timers\s(?P<keepalive>\d+)\s(?P<holdtime>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "timers {{ timers.keepalive }} {{ timers.holdtime }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "timers": {
                            "keepalive": "{{ keepalive }}",
                            "holdtime": "{{ holdtime }}",
                        },
                    },
                },
            },
        },
        {
            "name": "transport",
            "getval": re.compile(
                r"""
                transport\sconnection-mode
                \s(?P<passive>passive)
                $""", re.VERBOSE,
            ),
            "setval": "transport connection-mode passive",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "transport": {
                            "connection_mode": {
                                "passive": "{{ not not passive }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "ttl_security",
            "getval": re.compile(
                r"""
                ttl-security\shops\s(?P<hops>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "ttl-security hops {{ ttl_security.hops|string }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "ttl_security": {
                            "hops": "{{ hops }}",
                        },
                    },
                },
            },
        },
        {
            "name": "update_source",
            "getval": re.compile(
                r"""
                update-source\s(?P<update_source>\S+)
                $""", re.VERBOSE,
            ),
            "setval": "update-source {{ update_source }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "update_source": "{{ update_source }}",
                    },
                },
            },
        },
        # start template AF parsers
        {
            "name": "address_family",
            "getval": re.compile(
                r"""
                template\speer\s(?P<name>\S+)
                \saddress-family\s(?P<afi>\S+)\s(?P<safi>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "address-family {{ afi }}{{ (' ' + safi) if safi is defined else '' }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            "{{ afi + '_' + safi|d() }}": {
                                "afi": "{{ afi }}",
                                "safi": "{{ safi }}",
                            },
                        },
                    },
                },
            },
            "shared": True,
        },
        {
            "name": "advertise_map.exist_map",
            "getval": re.compile(
                r"""
                advertise-map
                \s(?P<route_map>\S+)
                \sexist-map\s(?P<exist_map>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "advertise-map {{ advertise_map.route_map }} exist-map {{ advertise_map.exist_map }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "advertise_map": {
                                    "route_map": "{{ route_map }}",
                                    "exist_map": "{{ exist_map }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "advertise_map.non_exist_map",
            "getval": re.compile(
                r"""
                advertise-map
                \s(?P<route_map>\S+)
                \snon-exist-map\s(?P<non_exist_map>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "advertise-map {{ advertise_map.route_map }} non-exist-map {{ advertise_map.non_exist_map }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "advertise_map": {
                                    "route_map": "{{ route_map }}",
                                    "non_exist_map": "{{ non_exist_map }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "advertisement_interval",
            "getval": re.compile(
                r"""
                advertisement-interval
                \s(?P<advertisement_interval>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "advertisement-interval {{ advertisement_interval }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "advertisement_interval": "{{ advertisement_interval }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "allowas_in",
            "getval": re.compile(
                r"""
                (?P<allowas_in>allowas-in)\s(?P<max_occurences>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "allowas-in{{ ' ' + allowas_in.max_occurences|string if allowas_in.max_occurences is defined else '' }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            "{{ afi + '_' + safi|d() }}": {
                                "allowas_in": {
                                    "set": "{{ True if allowas_in is defined and max_occurences is undefined }}",
                                    "max_occurences": "{{ max_occurences }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "as_override",
            "getval": re.compile(
                r"""
                (?P<as_override>as-override)
                $""",
                re.VERBOSE,
            ),
            "setval": "as-override",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            "{{ afi + '_' + safi|d() }}": {
                                "as_override": "{{ not not as_override }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "capability.additional_paths.receive",
            "getval": re.compile(
                r"""
                capability\sadditional-paths
                \s(?P<receive>receive)
                (\s(?P<disable>disable))?
                $""",
                re.VERBOSE,
            ),
            "setval": "capability additional-paths receive{{ ' disable' if capability.additional_paths.receive == 'disable' else '' }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            "{{ afi + '_' + safi|d() }}": {
                                "capability": {
                                    "additional_paths": {
                                        "receive": "{{ 'disable' if disable is defined else 'enable' }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "capability.additional_paths.send",
            "getval": re.compile(
                r"""
                capability\sadditional-paths
                \s(?P<send>send)
                (\s(?P<disable>disable))?
                $""",
                re.VERBOSE,
            ),
            "setval": "capability additional-paths send{{ ' disable' if capability.additional_paths.send == 'disable' else '' }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "capability": {
                                    "additional_paths": {
                                        "send": "{{ 'disable' if disable is defined else 'enable' }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "default_originate",
            "getval": re.compile(
                r"""
                (?P<default_originate>default-originate)
                (\sroute-map\s(?P<route_map>\S+))?
                $""",
                re.VERBOSE,
            ),
            "setval": "default-originate{{ ' route-map ' + default_originate.route_map if default_originate.route_map is defined else '' }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "default_originate": {
                                    "set": "{{ True if default_originate is defined and route_map is not defined }}",
                                    "route_map": "{{ route_map }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "disable_peer_as_check",
            "getval": re.compile(
                r"""
                (?P<disable_peer_as_check>disable-peer-as-check)
                $""",
                re.VERBOSE,
            ),
            "setval": "disable-peer-as-check",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "disable_peer_as_check": "{{ not not disable_peer_as_check }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "filter_list.inbound",
            "getval": re.compile(
                r"""
                filter-list
                \s(?P<in>\S+)\s(?:in)
                $""",
                re.VERBOSE,
            ),
            "setval": "filter-list {{ filter_list.inbound }} in",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "filter_list": {
                                    "inbound": "{{ in }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "filter_list.outbound",
            "getval": re.compile(
                r"""
                filter-list
                \s(?P<out>\S+)\s(?:out)
                $""",
                re.VERBOSE,
            ),
            "setval": "filter-list {{ filter_list.outbound }} out",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "filter_list": {
                                    "outbound": "{{ out }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "inherit.peer_policy",
            "getval": re.compile(
                r"""
                inherit\speer-policy
                \s(?P<peer_policy>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "inherit peer-policy {{ inherit.peer_policy }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "inherit": {
                                    "peer_policy": "{{ peer_policy }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "maximum_prefix",
            "getval": re.compile(
                r"""
                maximum-prefix
                \s(?P<max_prefix_limit>\d+)
                (\s(?P<generate_warning_threshold>\d+))?
                (\srestart\s(?P<restart_interval>\d+))?
                (\s(?P<warning_only>warning-only))?
                $""",
                re.VERBOSE,
            ),
            "setval": "maximum-prefix"
                      "{{ ' ' + maximum_prefix.max_prefix_limit|string if maximum_prefix.max_prefix_limit|d(None) else '' }}"
                      "{{ ' ' + maximum_prefix.generate_warning_threshold|string if maximum_prefix.generate_warning_threshold|d(None) else '' }}"
                      "{{ ' restart ' + maximum_prefix.restart_interval|string if maximum_prefix.restart_interval|d(None) else '' }}"
                      "{{ ' warning-only' if maximum_prefix.warning_only|d(False) else '' }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "maximum_prefix": {
                                    "max_prefix_limit": "{{ max_prefix_limit }}",
                                    "generate_warning_threshold": "{{ generate_warning_threshold }}",
                                    "restart_interval": "{{ restart_interval }}",
                                    "warning_only": "{{ not not warning_only }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "next_hop_self",
            "getval": re.compile(
                r"""
                (?P<next_hop_self>next-hop-self)
                (\s(?P<all_routes>all))?
                $""",
                re.VERBOSE,
            ),
            "setval": "next-hop-self{{ ' all' if next_hop_self.all_routes|d(False) else '' }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "next_hop_self": {
                                    "set": "{{ True if next_hop_self is defined and all_routes is not defined }}",
                                    "all_routes": "{{ not not all_routes }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "next_hop_third_party",
            "getval": re.compile(
                r"""
                no\s(?P<next_hop_third_party>next-hop-third-party)
                $""",
                re.VERBOSE,
            ),
            "setval": "next-hop-third-party",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "next_hop_third_party": "{{ not next_hop_third_party }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "prefix_list.inbound",
            "getval": re.compile(
                r"""
                prefix-list
                \s(?P<in>\S+)\s(?:in)
                $""",
                re.VERBOSE,
            ),
            "setval": "prefix-list {{ prefix_list.inbound }} in",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "prefix_list": {
                                    "inbound": "{{ in }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "prefix_list.outbound",
            "getval": re.compile(
                r"""
                prefix-list
                \s(?P<out>\S+)\s(?:out)
                $""",
                re.VERBOSE,
            ),
            "setval": "prefix-list {{ prefix_list.outbound }} out",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "prefix_list": {
                                    "outbound": "{{ out }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_map.inbound",
            "getval": re.compile(
                r"""
                route-map
                \s(?P<in>\S+)\s(?:in)
                $""",
                re.VERBOSE,
            ),
            "setval": "route-map {{ route_map.inbound }} in",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "route_map": {
                                    "inbound": "{{ in }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_map.outbound",
            "getval": re.compile(
                r"""
                route-map
                \s(?P<out>\S+)\s(?:out)
                $""",
                re.VERBOSE,
            ),
            "setval": "route-map {{ route_map.outbound }} out",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "route_map": {
                                    "outbound": "{{ out }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "route_reflector_client",
            "getval": re.compile(
                r"""
                (?P<route_reflector_client>route-reflector-client)
                $""",
                re.VERBOSE,
            ),
            "setval": "route-reflector-client",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "route_reflector_client": "{{ not not route_reflector_client }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "send_community_standard",
            "getval": re.compile(
                r"""
                (?P<send_community_std>send-community)
                $""",
                re.VERBOSE,
            ),
            "setval": "send-community",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "send_community_std": "{{ not not send_community_std }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "send_community_extended",
            "getval": re.compile(
                r"""
                (?P<send_community_ext>send-community\sextended)
                $""",
                re.VERBOSE,
            ),
            "setval": "send-community extended",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "send_community_ext": "{{ not not send_community_ext }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "soft_reconfiguration_inbound",
            "getval": re.compile(
                r"""
                (?P<soft_reconfiguration_inbound>soft-reconfiguration\sinbound)
                (\s(?P<always>always))?
                $""",
                re.VERBOSE,
            ),
            "setval": "soft-reconfiguration inbound{{ ' always' if soft_reconfiguration_inbound.always|d(False) else '' }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "soft_reconfiguration_inbound": {
                                    "set": "{{ True if soft_reconfiguration_inbound is defined and always is undefined }}",
                                    "always": "{{ not not always }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "soo",
            "getval": re.compile(
                r"""
                soo\s(?P<soo>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "soo {{ soo }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "soo": "{{ soo }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "suppress_inactive",
            "getval": re.compile(
                r"""
                (?P<suppress_inactive>suppress-inactive)
                $""",
                re.VERBOSE,
            ),
            "setval": "suppress-inactive",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "suppress_inactive": "{{ not not suppress_inactive }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "unsuppress_map",
            "getval": re.compile(
                r"""
                unsuppress-map\s(?P<unsuppress_map>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": "unsuppress-map {{ unsuppress_map }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "unsuppress_map": "{{ unsuppress_map }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "weight",
            "getval": re.compile(
                r"""
                weight\s(?P<weight>\d+)
                $""",
                re.VERBOSE,
            ),
            "setval": "weight {{ weight }}",
            "result": {
                "neighbor": {
                    "{{ name }}": {
                        "address_family": {
                            '{{ afi + "_" + safi|d() }}': {
                                "weight": "{{ weight }}",
                            },
                        },
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: networks_traffic_shaping_dscp_tagging_options_info
short_description: Information module for networks _trafficshaping _dscptaggingoptions
description:
- Get all networks _trafficshaping _dscptaggingoptions.
- Returns the available DSCP tagging options for your traffic shaping rules.
version_added: '2.16.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  networkId:
    description:
    - NetworkId path parameter. Network ID.
    type: str
requirements:
- meraki >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco Meraki documentation for networks getNetworkTrafficShapingDscpTaggingOptions
  description: Complete reference of the getNetworkTrafficShapingDscpTaggingOptions API.
  link: https://developer.cisco.com/meraki/api-v1/#!get-network-traffic-shaping-dscp-tagging-options
notes:
  - SDK Method used are
    networks.Networks.get_network_traffic_shaping_dscp_tagging_options,

  - Paths used are
    get /networks/{networkId}/trafficShaping/dscpTaggingOptions,
"""

EXAMPLES = r"""
- name: Get all networks _trafficshaping _dscptaggingoptions
  cisco.meraki.networks_traffic_shaping_dscp_tagging_options_info:
    meraki_api_key: "{{meraki_api_key}}"
    meraki_base_url: "{{meraki_base_url}}"
    meraki_single_request_timeout: "{{meraki_single_request_timeout}}"
    meraki_certificate_path: "{{meraki_certificate_path}}"
    meraki_requests_proxy: "{{meraki_requests_proxy}}"
    meraki_wait_on_rate_limit: "{{meraki_wait_on_rate_limit}}"
    meraki_nginx_429_retry_wait_time: "{{meraki_nginx_429_retry_wait_time}}"
    meraki_action_batch_retry_wait_time: "{{meraki_action_batch_retry_wait_time}}"
    meraki_retry_4xx_error: "{{meraki_retry_4xx_error}}"
    meraki_retry_4xx_error_wait_time: "{{meraki_retry_4xx_error_wait_time}}"
    meraki_maximum_retries: "{{meraki_maximum_retries}}"
    meraki_output_log: "{{meraki_output_log}}"
    meraki_log_file_prefix: "{{meraki_log_file_prefix}}"
    meraki_log_path: "{{meraki_log_path}}"
    meraki_print_console: "{{meraki_print_console}}"
    meraki_suppress_logging: "{{meraki_suppress_logging}}"
    meraki_simulate: "{{meraki_simulate}}"
    meraki_be_geo_id: "{{meraki_be_geo_id}}"
    meraki_use_iterator_for_get_pages: "{{meraki_use_iterator_for_get_pages}}"
    meraki_inherit_logging_config: "{{meraki_inherit_logging_config}}"
    networkId: string
  register: result

"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {}
    ]
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Sabari Jaganathan (@sajagana) <sajagana@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_schema_template_anp_epg_useg_attribute
short_description: Manage EPG uSeg Attributes in schema templates
description:
- Manage uSeg Attributes in the schema template EPGs on Cisco ACI Multi-Site.
author:
- Sabari Jaganathan (@sajagana)
options:
  schema:
    description:
    - The name of the Schema.
    type: str
    required: true
  template:
    description:
    - The name of the Template.
    type: str
    required: true
  anp:
    description:
    - The name of the Application Profile.
    type: str
    required: true
  epg:
    description:
    - The name of the EPG.
    type: str
    required: true
  name:
    description:
    - The name and display name of the uSeg Attribute.
    type: str
    aliases: [ useg ]
  description:
    description:
    - The description of the uSeg Attribute.
    type: str
    aliases: [ descr ]
  type:
    description:
    - The type of the uSeg Attribute.
    type: str
    choices: [ vm_name, ip, mac, vmm_domain, vm_operating_system, vm_tag, vm_hypervisor_identifier, dns, vm_datacenter, vm_identifier, vnic_dn ]
    aliases: [ attribute_type ]
  value:
    description:
    - The value of the uSeg Attribute.
    type: str
    aliases: [ attribute_value ]
  operator:
    description:
    - The operator type of the uSeg Attribute.
    type: str
    choices: [ equals, contains, starts_with, ends_with ]
  useg_subnet:
    description:
    - The uSeg Subnet can only be used when the I(attribute_type) is IP.
    - Use C(false) to set the custom uSeg Subnet IP address to the uSeg Attribute.
    - Use C(true) to set the uSeg Subnet IP address to 0.0.0.0.
    type: bool
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
notes:
- Due to restrictions of the MSO REST API concurrent modifications to EPG subnets can be dangerous and corrupt data.
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Add an uSeg attr with attribute_type - ip
  cisco.mso.mso_schema_template_anp_epg_useg_attribute:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    name: useg_attr_ip
    attribute_type: ip
    useg_subnet: false
    value: 10.0.0.0/24
    state: present
  delegate_to: localhost

- name: Query a specific EPG uSeg attr with name
  cisco.mso.mso_schema_template_anp_epg_useg_attribute:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    name: useg_attr_ip
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all EPG uSeg attrs
  cisco.mso.mso_schema_template_anp_epg_useg_attribute:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    state: query
  delegate_to: localhost
  register: query_result

- name: Remove a uSeg attr from an EPG with name
  cisco.mso.mso_schema_template_anp_epg_useg_attribute:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema 1
    template: Template 1
    anp: ANP 1
    epg: EPG 1
    name: useg_attr_ip
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec
from ansible_collections.cisco.mso.plugins.module_utils.constants import EPG_U_SEG_ATTR_TYPE_MAP, EPG_U_SEG_ATTR_OPERATOR_LIST
from ansible_collections.cisco.mso.plugins.module_utils.schema import MSOSchema


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type="str", required=True),
        template=dict(type="str", required=True),
        anp=dict(type="str", required=True),
        epg=dict(type="str", required=True),
        name=dict(type="str", aliases=["useg"]),
        description=dict(type="str", aliases=["descr"]),
        type=dict(type="str", aliases=["attribute_type"], choices=list(EPG_U_SEG_ATTR_TYPE_MAP.keys())),
        value=dict(type="str", aliases=["attribute_value"]),
        operator=dict(type="str", choices=EPG_U_SEG_ATTR_OPERATOR_LIST),
        useg_subnet=dict(type="bool"),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["name"]],
            ["state", "present", ["name", "type"]],
            ["useg_subnet", False, ["value"]],
        ],
    )

    schema = module.params.get("schema")
    template = module.params.get("template").replace(" ", "")
    anp = module.params.get("anp")
    epg = module.params.get("epg")
    name = module.params.get("name")
    description = module.params.get("description")
    attribute_type = module.params.get("type")
    value = module.params.get("value")
    operator = module.params.get("operator")
    useg_subnet = module.params.get("useg_subnet")
    state = module.params.get("state")
    mso = MSOModule(module)

    if state == "present":
        if attribute_type in ["mac", "dns"] and value is None:
            mso.fail_json(msg="Failed due to invalid 'value' and the attribute_type is: {0}.".format(attribute_type))
        elif attribute_type not in ["mac", "dns", "ip"] and (value is None or operator is None):
            mso.fail_json(msg="Failed due to invalid 'value' or 'operator' and the attribute_type is: {0}.".format(attribute_type))

    mso_schema = MSOSchema(mso, schema, template)
    mso_schema.set_template(template)
    mso_schema.set_template_anp(anp)
    mso_schema.set_template_anp_epg(epg)

    if mso_schema.schema_objects["template_anp_epg"].details.get("uSegEpg"):
        mso_schema.set_template_anp_epg_useg_attr(name, fail_module=False)
        if mso_schema.schema_objects["template_anp_epg_useg_attribute"] is not None:
            useg_attr_path = "/templates/{0}/anps/{1}/epgs/{2}/uSegAttrs/{3}".format(
                template, anp, epg, mso_schema.schema_objects["template_anp_epg_useg_attribute"].index
            )
            mso.existing = mso_schema.schema_objects["template_anp_epg_useg_attribute"].details
    else:
        mso.fail_json(msg="{0}: is not a valid uSeg EPG.".format(epg))

    if state == "query":
        if name is None:
            mso.existing = mso_schema.schema_objects["template_anp_epg"].details.get("uSegAttrs")
        elif not mso.existing:
            mso.fail_json(msg="The uSeg Attribute: {0} not found.".format(name))
        mso.exit_json()

    useg_attrs_path = "/templates/{0}/anps/{1}/epgs/{2}/uSegAttrs".format(template, anp, epg)
    ops = []

    mso.previous = mso.existing
    if state == "absent":
        if mso.existing:
            mso.existing = {}
            ops.append(dict(op="remove", path=useg_attr_path))

    if state == "present":
        if not mso.existing and description is None:
            description = name

        payload = dict(name=name, displayName=name, description=description, type=EPG_U_SEG_ATTR_TYPE_MAP[attribute_type], value=value)

        if attribute_type == "ip":
            if useg_subnet is False:
                payload["fvSubnet"] = True
            else:
                payload["fvSubnet"] = False
                payload["value"] = "0.0.0.0"

        mso.sanitize(payload, collate=True)

        if mso.existing:
            ops.append(dict(op="replace", path=useg_attr_path, value=mso.sent))
        else:
            ops.append(dict(op="add", path=useg_attrs_path + "/-", value=mso.sent))

        mso.existing = mso.proposed

    if not module.check_mode:
        mso.request(mso_schema.path, method="PATCH", data=ops)

    mso.exit_json()


if __name__ == "__main__":
    main()
